import json
from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel
from sinch.domains.verification.enums import VerificationMethod
from sinch.domains.verification.models import VerificationIdentity


class ReportPhoneCallVerificationDataTransformationMixin:
    def as_dict(self):
        request_data = super().as_dict()
        payload = {"method": request_data["method"], "callout": {}}

        if request_data.get("code"):
            payload["callout"]["code"] = request_data["code"]

        return payload


class ReportFlashCallVerificationDataTransformationMixin:
    def as_dict(self):
        request_data = super().as_dict()
        payload = {"method": request_data["method"], "flashCall": {}}

        if request_data.get("cli"):
            payload["flashCall"]["cli"] = request_data["cli"]

        return payload


class ReportSMSVerificationDataTransformationMixin:
    def as_dict(self):
        request_data = super().as_dict()
        payload = {"method": request_data["method"], "sms": {}}

        if request_data.get("code"):
            payload["sms"]["code"] = request_data["code"]

        if request_data.get("cli"):
            payload["sms"]["cli"] = request_data["cli"]

        return payload


@dataclass
class StartVerificationRequest(SinchRequestBaseModel):
    identity: VerificationIdentity
    reference: str
    custom: str


@dataclass
class StartSMSVerificationRequest(StartVerificationRequest):
    expiry: str
    code_type: str
    template: str
    method: str = VerificationMethod.SMS.value

    def as_dict(self):
        payload = super().as_dict()
        payload["smsOptions"] = {}

        if payload.get("code_type"):
            payload["smsOptions"].update({
                "codeType": payload.pop("code_type")
            })
        elif payload.get("expiry"):
            payload["smsOptions"].update({
                "expiry": payload.pop("expiry")
            })
        elif payload.get("template"):
            payload["smsOptions"].update({
                "template": payload.pop("template")
            })
        return payload


@dataclass
class StartFlashCallVerificationRequest(StartVerificationRequest):
    dial_timeout: int
    method: str = VerificationMethod.FLASH_CALL.value

    def as_dict(self):
        payload = super().as_dict()
        if payload.get("dial_timeout"):
            payload["flashCallOptions"] = {
                "dialTimeout": payload.pop("dial_timeout")
            }
        return payload


@dataclass
class StartPhoneCallVerificationRequest(StartVerificationRequest):
    method: str = VerificationMethod.CALLOUT.value


@dataclass
class StartCalloutVerificationRequest(StartVerificationRequest):
    speech_locale: str
    method: str = VerificationMethod.CALLOUT.value

    def as_dict(self):
        payload = super().as_dict()
        if payload.get("speech_locale"):
            payload["calloutOptions"] = {
                "speech": {
                    "locale": payload.pop("speech_locale")
                }
            }
        return payload


@dataclass
class StartDataVerificationRequest(StartVerificationRequest):
    method: str = VerificationMethod.SEAMLESS.value


@dataclass
class ReportVerificationByIdentityRequest(SinchRequestBaseModel):
    endpoint: str


@dataclass
class ReportVerificationByIdentityAndSMSRequest(
    ReportSMSVerificationDataTransformationMixin,
    ReportVerificationByIdentityRequest
):
    code: str
    cli: str
    method: str = VerificationMethod.SMS.value


@dataclass
class ReportVerificationByIdentityAndFlashCallRequest(
    ReportFlashCallVerificationDataTransformationMixin,
    ReportVerificationByIdentityRequest
):
    cli: str
    method: str = VerificationMethod.FLASH_CALL.value


@dataclass
class ReportVerificationByIdentityAndPhoneCallRequest(
    ReportPhoneCallVerificationDataTransformationMixin,
    ReportVerificationByIdentityRequest
):
    code: str
    method: str = VerificationMethod.CALLOUT.value


@dataclass
class ReportVerificationByIdRequest(SinchRequestBaseModel):
    id: str


@dataclass
class ReportVerificationByIdAndSMSRequest(
    ReportSMSVerificationDataTransformationMixin,
    ReportVerificationByIdRequest
):
    code: str
    cli: str
    method: str = VerificationMethod.SMS.value


@dataclass
class ReportVerificationByIdAndFlashCallRequest(
    ReportFlashCallVerificationDataTransformationMixin,
    ReportVerificationByIdRequest
):
    cli: str
    method: str = VerificationMethod.FLASH_CALL.value


@dataclass
class ReportVerificationByIdAndPhoneCallRequest(
    ReportPhoneCallVerificationDataTransformationMixin,
    ReportVerificationByIdRequest
):
    code: str
    method: str = VerificationMethod.CALLOUT.value


@dataclass
class GetVerificationStatusByReferenceRequest(SinchRequestBaseModel):
    reference: str


@dataclass
class GetVerificationStatusByIdentityRequest(SinchRequestBaseModel):
    endpoint: str
    method: str


@dataclass
class GetVerificationStatusByIdRequest(SinchRequestBaseModel):
    id: str


@dataclass
class ReportVerificationByIdentityRequestLegacy(ReportVerificationByIdentityRequest):
    verification_report_request: dict

    def as_json(self):
        return json.dumps(self.verification_report_request)


@dataclass
class ReportVerificationByIdRequestLegacy(ReportVerificationByIdRequest):
    verification_report_request: dict

    def as_json(self):
        return json.dumps(self.verification_report_request)



from logging import Logger
from sinch.core.clients.sinch_client_base import SinchClientBase
from sinch.core.clients.sinch_client_configuration import Configuration
from sinch.core.token_manager import TokenManagerAsync
from sinch.core.adapters.httpx_adapter import HTTPXTransport
from sinch.domains.authentication import AuthenticationAsync
from sinch.domains.numbers import NumbersAsync
from sinch.domains.conversation import ConversationAsync
from sinch.domains.sms import SMSAsync
from sinch.domains.verification import VerificationAsync
from sinch.domains.voice import VoiceAsync


class SinchClientAsync(SinchClientBase):
    """
    Asynchronous implementation of the Sinch Client
    By default this implementation uses HTTPTransportAioHTTP based on AioHTTP library
    Custom Async HTTPTransport implementation can be provided via `transport` argument
    """
    def __init__(
        self,
        key_id: str = None,
        key_secret: str = None,
        project_id: str = None,
        logger_name: str = None,
        logger: Logger = None,
        application_key: str = None,
        application_secret: str = None,
        service_plan_id: str = None,
        sms_api_token: str = None
    ):
        self.configuration = Configuration(
            key_id=key_id,
            key_secret=key_secret,
            project_id=project_id,
            logger_name=logger_name,
            logger=logger,
            transport=HTTPXTransport(self),
            token_manager=TokenManagerAsync(self),
            application_secret=application_secret,
            application_key=application_key,
            service_plan_id=service_plan_id,
            sms_api_token=sms_api_token
        )

        self.authentication = AuthenticationAsync(self)
        self.numbers = NumbersAsync(self)
        self.conversation = ConversationAsync(self)
        self.sms = SMSAsync(self)
        self.verification = VerificationAsync(self)
        self.voice = VoiceAsync(self)



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class RegisterConversationOptInRequest(SinchRequestBaseModel):
    request_id: str
    app_id: str
    channels: list
    recipient: dict
    processing_strategy: str


@dataclass
class RegisterConversationOptOutRequest(SinchRequestBaseModel):
    request_id: str
    app_id: str
    channels: list
    recipient: dict
    processing_strategy: str



from datetime import datetime
from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.voice.models import Price, Destination


@dataclass
class GetVoiceCallResponse(SinchBaseModel):
    from_: Destination
    to: Destination
    domain: str
    call_id: str
    duration: int
    status: str
    result: str
    reason: str
    timestamp: datetime
    custom: str
    user_rate: Price
    debit: Price


@dataclass
class UpdateVoiceCallResponse(SinchBaseModel):
    pass


class ManageVoiceCallResponse(SinchBaseModel):
    pass



from dataclasses import dataclass

from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class ListActiveNumbersRequest(SinchRequestBaseModel):
    region_code: str
    number_type: str
    page_size: int
    capabilities: list
    number_search_pattern: str
    number_pattern: str
    page_token: str


@dataclass
class GetNumberConfigurationRequest(SinchRequestBaseModel):
    phone_number: str


@dataclass
class UpdateNumberConfigurationRequest(SinchRequestBaseModel):
    phone_number: str
    display_name: str
    sms_configuration: dict
    voice_configuration: dict
    app_id: str


@dataclass
class ReleaseNumberFromProjectRequest(SinchRequestBaseModel):
    phone_number: str



from dataclasses import dataclass
from typing import List
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.conversation.models import SinchConversationMessage


@dataclass
class SendConversationMessageResponse(SinchBaseModel):
    accepted_time: str
    message_id: str


@dataclass
class ListConversationMessagesResponse(SinchBaseModel):
    messages: List[SinchConversationMessage]
    next_page_token: str


@dataclass
class GetConversationMessageResponse(SinchConversationMessage):
    pass


@dataclass
class DeleteConversationMessageResponse(SinchBaseModel):
    pass



from dataclasses import dataclass
from typing import List
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.voice.models import ConferenceParticipant


@dataclass
class GetVoiceConferenceResponse(SinchBaseModel):
    participants: List[ConferenceParticipant]


@dataclass
class KickAllVoiceConferenceResponse(SinchBaseModel):
    pass


@dataclass
class ManageParticipantVoiceConferenceResponse(SinchBaseModel):
    pass


@dataclass
class KickParticipantVoiceConferenceResponse(SinchBaseModel):
    pass



from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.sms.models.batches import Batch


@dataclass
class SendSMSBatchResponse(Batch):
    pass


@dataclass
class ReplaceSMSBatchResponse(Batch):
    pass


@dataclass
class ListSMSBatchesResponse(SinchBaseModel):
    page: str
    page_size: str
    count: str
    batches: list


@dataclass
class GetSMSBatchResponse(Batch):
    pass


@dataclass
class CancelSMSBatchResponse(Batch):
    pass


@dataclass
class SendSMSBatchDryRunResponse(SinchBaseModel):
    number_of_recipients: int
    number_of_messages: int
    per_recipient: list


@dataclass
class UpdateSMSBatchResponse(SinchBaseModel):
    id: str
    to: list
    from_: str
    body: str
    campaign_id: str
    delivery_report: str
    send_at: str
    expire_at: str
    callback_url: str
    cancelled: bool
    type: str
    created_at: str
    modified_at: str


@dataclass
class SendSMSDeliveryFeedbackResponse(SinchBaseModel):
    pass



""" Sinch Python SDK
To access Sinch resources, use the Sync or Async version of the Sinch Client.
"""
__version__ = "1.1.0"

from sinch.core.clients.sinch_client_sync import SinchClient
from sinch.core.clients.sinch_client_async import SinchClientAsync

__all__ = (SinchClient, SinchClientAsync)



from typing import List

from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.voice.models import ApplicationNumber, Price


@dataclass
class GetNumbersVoiceApplicationResponse(SinchBaseModel):
    numbers: List[ApplicationNumber]


@dataclass
class AssignNumbersVoiceApplicationResponse(SinchBaseModel):
    pass


@dataclass
class UnassignNumbersVoiceApplicationResponse(SinchBaseModel):
    pass


@dataclass
class UpdateCallbackUrlsVoiceApplicationResponse(SinchBaseModel):
    pass


@dataclass
class GetCallbackUrlsVoiceApplicationResponse(SinchBaseModel):
    primary: str
    fallback: str


@dataclass
class QueryNumberVoiceApplicationResponse(SinchBaseModel):
    country_id: str
    number_type: str
    normalized_number: str
    restricted: bool
    rate: Price



from dataclasses import dataclass

from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class ListAvailableNumbersRequest(SinchRequestBaseModel):
    region_code: str
    number_type: str
    page_size: int
    capabilities: list
    number_search_pattern: str
    number_pattern: str


@dataclass
class ActivateNumberRequest(SinchRequestBaseModel):
    phone_number: str
    sms_configuration: dict
    voice_configuration: dict


@dataclass
class RentAnyNumberRequest(SinchRequestBaseModel):
    region_code: str
    type_: str
    number_pattern: str
    capabilities: list
    sms_configuration: dict
    voice_configuration: dict
    callback_url: str


@dataclass
class CheckNumberAvailabilityRequest(SinchRequestBaseModel):
    phone_number: str



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.delivery_reports import DeliveryReport
from sinch.domains.sms.models.delivery_reports.requests import ListSMSDeliveryReportsRequest
from sinch.domains.sms.models.delivery_reports.responses import ListSMSDeliveryReportsResponse


class ListDeliveryReportsEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/delivery_reports"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: ListSMSDeliveryReportsRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id
        )

    def build_query_params(self):
        return self.request_data.as_dict()

    def handle_response(self, response: HTTPResponse):
        super(ListDeliveryReportsEndpoint, self).handle_response(response)
        return ListSMSDeliveryReportsResponse(
            delivery_reports=[
                DeliveryReport(
                    at=delivery_report.get("at"),
                    batch_id=delivery_report.get("batch_id"),
                    code=delivery_report.get("code"),
                    recipient=delivery_report.get("recipient"),
                    status=delivery_report.get("status"),
                    applied_originator=delivery_report.get("applied_originator"),
                    client_reference=delivery_report.get("client_reference"),
                    encoding=delivery_report.get("encoding"),
                    number_of_message_parts=delivery_report.get("number_of_message_parts"),
                    operator=delivery_report.get("operator"),
                    operator_status_at=delivery_report.get("operator_status_at"),
                    type=delivery_report.get("type")
                ) for delivery_report in response.body["delivery_reports"]
            ],
            page=response.body.get("page"),
            page_size=response.body.get("page_size"),
            count=response.body.get("count")
        )



from .instructions import (
    TranscriptionOptions, RecordingOptions, PlayFileInstruction,
    SayInstruction, SendDtmfInstruction, SetCookieInstruction,
    AnswerInstruction, StartRecordingInstruction, StopRecordingInstruction
)

__all__ = [
    "TranscriptionOptions", "RecordingOptions", "PlayFileInstruction",
    "SayInstruction", "SendDtmfInstruction", "SetCookieInstruction",
    "AnswerInstruction", "StartRecordingInstruction", "StopRecordingInstruction"
]



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class ListSMSInboundMessageRequest(SinchRequestBaseModel):
    start_date: str
    to: str
    end_date: str
    page_size: int
    page_size: int
    client_reference: str
    page: int = 0


@dataclass
class GetSMSInboundMessageRequest(SinchRequestBaseModel):
    inbound_id: str



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.contact.responses import ListConversationContactsResponse
from sinch.domains.conversation.models import SinchConversationContact


class ListContactsEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/contacts"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id, request_data):
        super(ListContactsEndpoint, self).__init__(project_id, request_data)
        self.request_data = request_data
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id
        )

    def build_query_params(self):
        params = {}
        if self.request_data.page_size:
            params["page_size"] = self.request_data.page_size

        if self.request_data.page_token:
            params["page_token"] = self.request_data.page_token

        return params

    def handle_response(self, response: HTTPResponse) -> ListConversationContactsResponse:
        super(ListContactsEndpoint, self).handle_response(response)
        return ListConversationContactsResponse(
            contacts=[SinchConversationContact(
                id=contact["id"],
                channel_identities=contact["channel_identities"],
                channel_priority=contact["channel_priority"],
                display_name=contact["display_name"],
                email=contact["email"],
                external_id=contact["external_id"],
                metadata=contact["metadata"],
                language=contact["language"]
            ) for contact in response.body["contacts"]],
            next_page_token=response.body.get("next_page_token")
        )



from dataclasses import dataclass
from typing import Optional, List, TypedDict
from sinch.core.models.base_model import SinchRequestBaseModel
from sinch.domains.voice.models import Destination, ConferenceDTMFOptions


class Action(SinchRequestBaseModel):
    name: str


class AnsweringMachineDetection(TypedDict):
    enabled: bool


class CallHeader(TypedDict):
    key: str
    value: str


@dataclass
class HangupAction(Action):
    name: str = "hangup"


@dataclass
class ContinueAction(Action):
    name: str = "continue"


@dataclass
class ConnectPstnAction(Action):
    name: str = "connectPstn"
    number: Optional[str] = None
    locale: Optional[str] = None
    max_duration: Optional[int] = None
    dial_timeout: Optional[int] = None
    cli: Optional[str] = None
    suppress_callbacks: Optional[bool] = None
    dtmf: Optional[str] = None
    indications: Optional[str] = None
    amd: Optional[AnsweringMachineDetection] = None

    def as_dict(self):
        payload = super().as_dict()
        if payload.get("max_duration"):
            payload["maxDuration"] = payload.pop("max_duration")

        if payload.get("dial_timeout"):
            payload["dialTimeout"] = payload.pop("dial_timeout")

        if payload.get("suppress_callbacks"):
            payload["suppressCallbacks"] = payload.pop("suppress_callbacks")

        return payload


@dataclass
class ConnectMxpAction(Action):
    name: str = "connectMxp"
    destination: Optional[Destination] = None
    call_headers: Optional[List[CallHeader]] = None

    def as_dict(self):
        payload = super().as_dict()
        if payload.get("call_headers"):
            payload["callHeaders"] = payload.pop("call_headers")

        return payload


@dataclass
class Option(SinchRequestBaseModel):
    dtmf: str
    action: str


@dataclass
class MenuOption(SinchRequestBaseModel):
    id: str
    main_prompt: Optional[str] = None
    repeat_prompt: Optional[str] = None
    repeats: Optional[int] = None
    max_digits: Optional[int] = None
    timeout_mills: Optional[int] = None
    max_timeout_mills: Optional[int] = None
    options: Optional[List[Option]] = None

    def as_dict(self):
        payload = super().as_dict()
        if payload.get("main_prompt"):
            payload["mainPrompt"] = payload.pop("main_prompt")

        if payload.get("repeat_prompt"):
            payload["repeatPrompt"] = payload.pop("repeat_prompt")

        if payload.get("max_digits"):
            payload["maxDigits"] = payload.pop("max_digits")

        if payload.get("timeout_mills"):
            payload["timeoutMills"] = payload.pop("timeout_mills")

        if payload.get("max_timeout_mills"):
            payload["maxTimeoutMills"] = payload.pop("max_timeout_mills")

        return payload


@dataclass
class ConnectSipAction(Action):
    destination: Optional[Destination]
    name: str = "connectSip"
    max_duration: Optional[int] = None
    cli: Optional[str] = None
    transport: Optional[str] = None
    suppress_callbacks: Optional[bool] = None
    call_headers: Optional[List[CallHeader]] = None
    moh: Optional[str] = None

    def as_dict(self):
        payload = super().as_dict()
        if payload.get("max_duration"):
            payload["maxDuration"] = payload.pop("max_duration")

        if payload.get("suppress_callbacks"):
            payload["suppressCallbacks"] = payload.pop("suppress_callbacks")

        if payload.get("call_headers"):
            payload["callHeaders"] = payload.pop("call_headers")

        return payload


@dataclass
class ConnectConfAction(Action):
    conference_id: str
    name: str = "connectConf"
    conference_dtmf_options: Optional[ConferenceDTMFOptions] = None
    moh: Optional[str] = None

    def as_dict(self):
        payload = super().as_dict()
        if payload.get("conference_id"):
            payload["conferenceId"] = payload.pop("conference_id")

        if payload.get("conference_dtmf_options"):
            payload["conferenceDtmfOptions"] = payload.pop("conference_dtmf_options")

        return payload


@dataclass
class RunMenuAction(Action):
    name: str = "runMenu"
    barge: Optional[bool] = None
    locale: Optional[str] = None
    main_menu: Optional[str] = None
    enable_voice: Optional[bool] = None
    menus: Optional[List[MenuOption]] = None

    def as_dict(self):
        payload = super().as_dict()
        if payload.get("main_menu"):
            payload["mainMenu"] = payload.pop("main_menu")

        if payload.get("enable_voice"):
            payload["enableVoice"] = payload.pop("enable_voice")

        return payload


@dataclass
class ParkAction(Action):
    name: str = "park"
    locale: Optional[str] = None
    intro_prompt: Optional[str] = None
    hold_prompt: Optional[str] = None
    max_duration: Optional[int] = None

    def as_dict(self):
        payload = super().as_dict()
        if payload.get("intro_prompt"):
            payload["introPrompt"] = payload.pop("intro_prompt")

        if payload.get("hold_prompt"):
            payload["holdPrompt"] = payload.pop("hold_prompt")

        if payload.get("max_duration"):
            payload["maxDuration"] = payload.pop("max_duration")

        return payload



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class ListSMSDeliveryReportsRequest(SinchRequestBaseModel):
    code: str
    status: str
    start_date: str
    end_date: str
    client_reference: str
    page_size: int
    page: int = 0


@dataclass
class GetSMSDeliveryReportForBatchRequest(SinchRequestBaseModel):
    batch_id: str
    type_: str
    status: list
    code: list


@dataclass
class GetSMSDeliveryReportForNumberRequest(SinchRequestBaseModel):
    batch_id: str
    recipient_number: str



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.app.responses import ListConversationAppsResponse
from sinch.domains.conversation.models import SinchConversationApp


class ListAppsEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/apps"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str):
        super(ListAppsEndpoint, self).__init__(project_id, request_data=None)
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id
        )

    def handle_response(self, response: HTTPResponse) -> ListConversationAppsResponse:
        super(ListAppsEndpoint, self).handle_response(response)
        return ListConversationAppsResponse(
            apps=[
                SinchConversationApp(
                    id=contact["id"],
                    channel_credentials=contact["channel_credentials"],
                    processing_mode=contact["processing_mode"],
                    conversation_metadata_report_view=contact["conversation_metadata_report_view"],
                    display_name=contact["display_name"],
                    rate_limits=contact["rate_limits"],
                    retention_policy=contact["retention_policy"],
                    dispatch_retention_policy=contact["dispatch_retention_policy"],
                    smart_conversation=contact["smart_conversation"]
                ) for contact in response.body["apps"]
            ]
        )



from logging import Logger
from sinch.core.clients.sinch_client_base import SinchClientBase
from sinch.core.clients.sinch_client_configuration import Configuration
from sinch.core.token_manager import TokenManager
from sinch.core.adapters.requests_http_transport import HTTPTransportRequests
from sinch.domains.authentication import Authentication
from sinch.domains.numbers import Numbers
from sinch.domains.conversation import Conversation
from sinch.domains.sms import SMS
from sinch.domains.verification import Verification
from sinch.domains.voice import Voice


class SinchClient(SinchClientBase):
    """
    Synchronous implementation of the Sinch Client
    By default this implementation uses HTTPTransportRequests based on Requests library
    Custom Sync HTTPTransport implementation can be provided via `transport` argument
    """
    def __init__(
        self,
        key_id: str = None,
        key_secret: str = None,
        project_id: str = None,
        logger_name: str = None,
        logger: Logger = None,
        application_key: str = None,
        application_secret: str = None,
        service_plan_id: str = None,
        sms_api_token: str = None
    ):
        self.configuration = Configuration(
            key_id=key_id,
            key_secret=key_secret,
            project_id=project_id,
            logger_name=logger_name,
            logger=logger,
            transport=HTTPTransportRequests(self),
            token_manager=TokenManager(self),
            application_key=application_key,
            application_secret=application_secret,
            service_plan_id=service_plan_id,
            sms_api_token=sms_api_token
        )

        self.authentication = Authentication(self)
        self.numbers = Numbers(self)
        self.conversation = Conversation(self)
        self.sms = SMS(self)
        self.verification = Verification(self)
        self.voice = Voice(self)



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class SendConversationEventRequest(SinchRequestBaseModel):
    app_id: str
    recipient: dict
    event: dict
    callback_url: str
    channel_priority_order: str
    event_metadata: str
    queue: str



import logging
from logging import Logger
from typing import Union

from sinch.core.ports.http_transport import HTTPTransport
from sinch.core.token_manager import TokenManager, TokenManagerAsync
from sinch.core.enums import HTTPAuthentication


class Configuration:
    """
    Sinch client configuration object.
    """
    def __init__(
        self,
        key_id: str,
        key_secret: str,
        project_id: str,
        transport: HTTPTransport,
        token_manager: Union[TokenManager, TokenManagerAsync],
        logger: Logger = None,
        logger_name: str = None,
        disable_https=False,
        connection_timeout=10,
        application_key: str = None,
        application_secret: str = None,
        service_plan_id: str = None,
        sms_api_token: str = None
    ):
        self.key_id = key_id
        self.key_secret = key_secret
        self.project_id = project_id
        self.application_key = application_key
        self.application_secret = application_secret
        self.connection_timeout = connection_timeout
        self.sms_api_token = sms_api_token
        self.service_plan_id = service_plan_id
        self.auth_origin = "auth.sinch.com"
        self.numbers_origin = "numbers.api.sinch.com"
        self.verification_origin = "verification.api.sinch.com"
        self.voice_applications_origin = "callingapi.sinch.com"
        self._voice_domain = "{}.api.sinch.com"
        self._voice_region = None
        self._conversation_region = "eu"
        self._conversation_domain = ".conversation.api.sinch.com"
        self._sms_region = "us"
        self._sms_region_with_service_plan_id = "us"
        self._sms_domain = "zt.{}.sms.api.sinch.com"
        self._sms_domain_with_service_plan_id = "{}.sms.api.sinch.com"
        self._sms_authentication = HTTPAuthentication.OAUTH.value
        self._templates_region = "eu"
        self._templates_domain = ".template.api.sinch.com"
        self.token_manager = token_manager
        self.disable_https = disable_https
        self.transport: HTTPTransport = transport

        self._set_conversation_origin()
        self._set_sms_origin()
        self._set_sms_origin_with_service_plan_id()
        self._set_templates_origin()
        self._set_voice_origin()

        if logger_name:
            self.logger = logging.getLogger(logger_name)
        elif logger:
            self.logger = logger
        else:
            self.logger = logging.getLogger("Sinch")

    def _set_sms_origin_with_service_plan_id(self):
        self.sms_origin_with_service_plan_id = self._sms_domain_with_service_plan_id.format(
            self._sms_region_with_service_plan_id
        )

    def _set_sms_region_with_service_plan_id(self, region):
        self._sms_region_with_service_plan_id = region
        self._set_sms_origin_with_service_plan_id()

    def _get_sms_region_with_service_plan_id(self):
        return self._sms_region_with_service_plan_id

    sms_region_with_service_plan_id = property(
        _get_sms_region_with_service_plan_id,
        _set_sms_region_with_service_plan_id,
        doc="SMS Region for service plan id version of the SMS API"
    )

    def _set_sms_domain_with_service_plan_id(self, domain):
        self._sms_domain_with_service_plan_id = domain
        self._set_sms_origin_with_service_plan_id()

    def _get_sms_domain_with_service_plan_id(self):
        return self._sms_domain_with_service_plan_id

    sms_domain_with_service_plan_id = property(
        _get_sms_domain_with_service_plan_id,
        _set_sms_domain_with_service_plan_id,
        doc="SMS Domain for service plan id version of the SMS API"
    )

    def _set_sms_origin(self):
        self.sms_origin = self._sms_domain.format(self._sms_region)

    def _set_sms_region(self, region):
        self._sms_region = region
        self._set_sms_origin()

    def _get_sms_region(self):
        return self._sms_region

    sms_region = property(
        _get_sms_region,
        _set_sms_region,
        doc="SMS Region"
    )

    def _set_sms_domain(self, domain):
        self._sms_domain = domain
        self._set_sms_origin()

    def _get_sms_domain(self):
        return self._sms_domain

    sms_domain = property(
        _get_sms_domain,
        _set_sms_domain,
        doc="SMS Domain"
    )

    def _set_conversation_origin(self):
        self.conversation_origin = self._conversation_region + self._conversation_domain

    def _set_conversation_region(self, region):
        self._conversation_region = region
        self._set_conversation_origin()

    def _get_conversation_region(self):
        return self._conversation_region

    conversation_region = property(
        _get_conversation_region,
        _set_conversation_region,
        doc="ConversationAPI Region"
    )

    def _set_conversation_domain(self, domain):
        self._conversation_domain = domain
        self._set_conversation_origin()

    def _get_conversation_domain(self):
        return self._conversation_domain

    conversation_domain = property(
        _get_conversation_domain,
        _set_conversation_domain,
        doc="ConversationAPI Domain"
    )

    def _set_templates_origin(self):
        self.templates_origin = self._templates_region + self._templates_domain

    def _set_templates_region(self, region):
        self._templates_region = region
        self._set_templates_origin()

    def _get_templates_region(self):
        return self._templates_region

    templates_region = property(
        _get_templates_region,
        _set_templates_region,
        doc="Conversation API Templates Region"
    )

    def _set_templates_domain(self, domain):
        self._templates_domain = domain
        self._set_templates_origin()

    def _get_templates_domain(self):
        return self._templates_domain

    templates_domain = property(
        _get_templates_domain,
        _set_templates_domain,
        doc="Conversation API Templates Domain"
    )

    def _set_voice_origin(self):
        if not self._voice_region:
            self.voice_origin = self._voice_domain.format("calling")
        else:
            self.voice_origin = self._voice_domain.format("calling-" + self._voice_region)

    def _set_voice_region(self, region):
        self._voice_region = region
        self._set_voice_origin()

    def _get_voice_region(self):
        return self._voice_region

    voice_region = property(
        _get_voice_region,
        _set_voice_region,
        doc="Voice Region"
    )



from dataclasses import dataclass

from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class UpdateNumbersCallbackConfigurationRequest(SinchRequestBaseModel):
    hmac_secret: str



from dataclasses import dataclass
from typing import List
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.conversation.models import (
    SinchConversationApp
)


@dataclass
class CreateConversationAppResponse(SinchConversationApp):
    pass


@dataclass
class DeleteConversationAppResponse(SinchBaseModel):
    pass


@dataclass
class ListConversationAppsResponse(SinchBaseModel):
    apps: List[SinchConversationApp]


@dataclass
class GetConversationAppResponse(SinchConversationApp):
    pass


@dataclass
class UpdateConversationAppResponse(SinchConversationApp):
    pass



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.app.responses import UpdateConversationAppResponse
from sinch.domains.conversation.models.app.requests import UpdateConversationAppRequest


class UpdateConversationAppEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/apps/{app_id}"
    HTTP_METHOD = HTTPMethods.PATCH.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: UpdateConversationAppRequest):
        super(UpdateConversationAppEndpoint, self).__init__(project_id, request_data)
        self.project_id = project_id
        self.request_data = request_data

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id,
            app_id=self.request_data.app_id
        )

    def build_query_params(self):
        if self.request_data.update_mask:
            return {"update_mask.paths": self.request_data.update_mask}

    def request_body(self):
        self.request_data.update_mask = None
        self.request_data.app_id = None
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse) -> UpdateConversationAppResponse:
        super(UpdateConversationAppEndpoint, self).handle_response(response)
        return UpdateConversationAppResponse(
            id=response.body["id"],
            channel_credentials=response.body["channel_credentials"],
            processing_mode=response.body["processing_mode"],
            conversation_metadata_report_view=response.body["conversation_metadata_report_view"],
            display_name=response.body["display_name"],
            rate_limits=response.body["rate_limits"],
            retention_policy=response.body["retention_policy"],
            dispatch_retention_policy=response.body["dispatch_retention_policy"],
            smart_conversation=response.body["smart_conversation"]
        )



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class QueryConversationCapabilityRequest(SinchRequestBaseModel):
    app_id: str
    recipient: dict
    request_id: str



from dataclasses import dataclass
from typing import List
from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.sms.models.groups import SMSGroup


@dataclass
class CreateSMSGroupResponse(SMSGroup):
    pass


@dataclass
class GetSMSGroupResponse(SMSGroup):
    pass


@dataclass
class SinchListSMSGroupResponse(SinchBaseModel):
    page: int
    page_size: int
    count: int
    groups: List[SMSGroup]


@dataclass
class SinchDeleteSMSGroupResponse(SinchBaseModel):
    pass


@dataclass
class SinchGetSMSGroupPhoneNumbersResponse(SinchBaseModel):
    phone_numbers: list


@dataclass
class UpdateSMSGroupResponse(SMSGroup):
    pass


@dataclass
class ReplaceSMSGroupResponse(SMSGroup):
    pass



from copy import deepcopy
from sinch.core.models.http_response import HTTPResponse
from sinch.domains.voice.endpoints.voice_endpoint import VoiceEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.voice.models.calls.responses import ManageVoiceCallResponse
from sinch.domains.voice.models.calls.requests import ManageVoiceCallRequest


class ManageCallEndpoint(VoiceEndpoint):
    ENDPOINT_URL = "{origin}/calling/v1/calls/id/{call_id}/leg/{call_leg}"
    HTTP_METHOD = HTTPMethods.PATCH.value
    HTTP_AUTHENTICATION = HTTPAuthentication.SIGNED.value

    def __init__(self, request_data: ManageVoiceCallRequest):
        self.request_data = request_data

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.voice_origin,
            call_id=self.request_data.call_id,
            call_leg=self.request_data.call_leg
        )

    def request_body(self):
        request_data = deepcopy(self.request_data)
        request_data.call_leg = None
        request_data.call_id = None
        return request_data.as_json()

    def handle_response(self, response: HTTPResponse) -> ManageVoiceCallResponse:
        super().handle_response(response)
        return ManageVoiceCallResponse()



from enum import Enum


class VerificationMethod(Enum):
    SMS = "sms"
    FLASH_CALL = "flashCall"
    CALLOUT = "callout"
    SEAMLESS = "seamless"


class VerificationStatus(Enum):
    PENDING = "PENDING"
    SUCCESSFUL = "SUCCESSFUL"
    FAIL = "FAIL"
    DENIED = "DENIED"
    ABORTED = "ABORTED"
    ERROR = "ERROR"



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.voice.endpoints.voice_endpoint import VoiceEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.voice.models.applications.requests import AssignNumbersVoiceApplicationRequest
from sinch.domains.voice.models.applications.responses import AssignNumbersVoiceApplicationResponse


class AssignVoiceNumbersEndpoint(VoiceEndpoint):
    ENDPOINT_URL = "{origin}/v1/configuration/numbers"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.SIGNED.value

    def __init__(self, request_data: AssignNumbersVoiceApplicationRequest):
        self.request_data = request_data

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.voice_applications_origin
        )

    def request_body(self):
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse) -> AssignNumbersVoiceApplicationResponse:
        super().handle_response(response)
        return AssignNumbersVoiceApplicationResponse()



from sinch.core.models.http_response import HTTPResponse
from sinch.core.endpoint import HTTPEndpoint
from sinch.domains.conversation.exceptions import ConversationException


class ConversationEndpoint(HTTPEndpoint):
    def handle_response(self, response: HTTPResponse):
        if response.status_code >= 400:
            raise ConversationException(
                message=response.body["error"].get("message"),
                response=response,
                is_from_server=True
            )



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models import SinchConversationMessage
from sinch.domains.conversation.models.message.responses import ListConversationMessagesResponse
from sinch.domains.conversation.models.message.requests import ListConversationMessagesRequest


class ListConversationMessagesEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/messages"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: ListConversationMessagesRequest):
        super(ListConversationMessagesEndpoint, self).__init__(project_id, request_data)
        self.request_data = request_data
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id
        )

    def build_query_params(self):
        query_params = {}
        if self.request_data.conversation_id:
            query_params["conversation_id"] = self.request_data.conversation_id

        if self.request_data.contact_id:
            query_params["contact_id"] = self.request_data.contact_id

        if self.request_data.page_size:
            query_params["page_size"] = self.request_data.page_size

        if self.request_data.page_token:
            query_params["page_token"] = self.request_data.page_token

        if self.request_data.app_id:
            query_params["app_id"] = self.request_data.app_id

        if self.request_data.view:
            query_params["view"] = self.request_data.view

        if self.request_data.messages_source:
            query_params["messages_source"] = self.request_data.messages_source

        if self.request_data.only_recipient_originated:
            query_params["only_recipient_originated"] = self.request_data.only_recipient_originated

        return query_params

    def handle_response(self, response: HTTPResponse) -> ListConversationMessagesResponse:
        super(ListConversationMessagesEndpoint, self).handle_response(response)
        return ListConversationMessagesResponse(
            messages=[
                SinchConversationMessage(
                    id=message["id"],
                    direction=message["direction"],
                    channel_identity=message["channel_identity"],
                    app_message=message["app_message"],
                    conversation_id=message["conversation_id"],
                    contact_id=message["contact_id"],
                    metadata=message["metadata"],
                    accept_time=message["accept_time"],
                    sender_id=message["sender_id"],
                    processing_mode=message["processing_mode"]
                ) for message in response.body["messages"]
            ],
            next_page_token=response.body.get("next_page_token")
        )



from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel


@dataclass
class InboundMessage(SinchBaseModel):
    type: str
    id: str
    from_: str
    to: str
    body: str
    operator_id: str
    send_at: str
    received_at: str
    client_reference: str



from sinch.core.pagination import IntBasedPaginator
from sinch.core.pagination import AsyncIntBasedPaginator

from sinch.domains.sms.endpoints.batches.send_batch import SendBatchSMSEndpoint
from sinch.domains.sms.endpoints.batches.list_batches import ListSMSBatchesEndpoint
from sinch.domains.sms.endpoints.batches.get_batch import GetSMSEndpoint
from sinch.domains.sms.endpoints.batches.cancel_batch import CancelBatchEndpoint
from sinch.domains.sms.endpoints.batches.update_batch import UpdateBatchSMSEndpoint
from sinch.domains.sms.endpoints.batches.replace_batch import ReplaceBatchSMSEndpoint
from sinch.domains.sms.endpoints.batches.send_delivery_feedback import SendDeliveryReportEndpoint
from sinch.domains.sms.endpoints.batches.send_batch_dry_run import SendBatchSMSDryRunEndpoint

from sinch.domains.sms.endpoints.groups.create_group import CreateSMSGroupEndpoint
from sinch.domains.sms.endpoints.groups.list_groups import ListSMSGroupEndpoint
from sinch.domains.sms.endpoints.groups.delete_group import DeleteSMSGroupEndpoint
from sinch.domains.sms.endpoints.groups.get_group import GetSMSGroupEndpoint
from sinch.domains.sms.endpoints.groups.update_group import UpdateSMSGroupEndpoint
from sinch.domains.sms.endpoints.groups.replace_group import ReplaceSMSGroupEndpoint
from sinch.domains.sms.endpoints.groups.get_phone_numbers_for_group import GetSMSGroupPhoneNumbersEndpoint

from sinch.domains.sms.endpoints.inbounds.list_incoming_messages import ListInboundMessagesEndpoint
from sinch.domains.sms.endpoints.inbounds.get_incoming_message import GetInboundMessagesEndpoint

from sinch.domains.sms.endpoints.delivery_reports.get_delivery_report_for_number import (
    GetDeliveryReportForNumberEndpoint
)
from sinch.domains.sms.endpoints.delivery_reports.get_delivery_report_for_batch import GetDeliveryReportForBatchEndpoint
from sinch.domains.sms.endpoints.delivery_reports.get_all_delivery_reports_for_project import (
    ListDeliveryReportsEndpoint
)

from sinch.domains.sms.models.batches.requests import (
    SendBatchRequest,
    ListBatchesRequest,
    GetBatchRequest,
    BatchDryRunRequest,
    CancelBatchRequest,
    UpdateBatchRequest,
    ReplaceBatchRequest,
    SendDeliveryFeedbackRequest
)

from sinch.domains.sms.models.batches.responses import (
    SendSMSBatchResponse,
    GetSMSBatchResponse,
    CancelSMSBatchResponse,
    SendSMSDeliveryFeedbackResponse,
    ListSMSBatchesResponse,
    UpdateSMSBatchResponse,
    ReplaceSMSBatchResponse,
    SendSMSBatchDryRunResponse
)

from sinch.domains.sms.models.groups.requests import (
    CreateSMSGroupRequest,
    ListSMSGroupRequest,
    DeleteSMSGroupRequest,
    GetSMSGroupRequest,
    GetSMSGroupPhoneNumbersRequest,
    UpdateSMSGroupRequest,
    ReplaceSMSGroupPhoneNumbersRequest
)

from sinch.domains.sms.models.groups.responses import (
    CreateSMSGroupResponse,
    SinchDeleteSMSGroupResponse,
    UpdateSMSGroupResponse,
    SinchListSMSGroupResponse,
    ReplaceSMSGroupResponse,
    GetSMSGroupResponse,
    SinchGetSMSGroupPhoneNumbersResponse
)

from sinch.domains.sms.models.inbounds.requests import (
    ListSMSInboundMessageRequest,
    GetSMSInboundMessageRequest
)

from sinch.domains.sms.models.inbounds.responses import (
    SinchListInboundMessagesResponse,
    GetInboundMessagesResponse
)

from sinch.domains.sms.models.delivery_reports.requests import (
    ListSMSDeliveryReportsRequest,
    GetSMSDeliveryReportForBatchRequest,
    GetSMSDeliveryReportForNumberRequest
)

from sinch.domains.sms.models.delivery_reports.responses import (
    ListSMSDeliveryReportsResponse,
    GetSMSDeliveryReportForBatchResponse,
    GetSMSDeliveryReportForNumberResponse
)


class SMSDeliveryReports:
    def __init__(self, sinch):
        self._sinch = sinch

    def list(
        self,
        page: int = 0,
        start_date: str = None,
        end_date: str = None,
        status: str = None,
        code: str = None,
        page_size: int = None,
        client_reference: str = None
    ) -> ListSMSDeliveryReportsResponse:
        return IntBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListDeliveryReportsEndpoint(
                sinch=self._sinch,
                request_data=ListSMSDeliveryReportsRequest(
                    page=page,
                    page_size=page_size,
                    start_date=start_date,
                    end_date=end_date,
                    status=status,
                    code=code,
                    client_reference=client_reference
                )
            )
        )

    def get_for_batch(
        self,
        batch_id: str,
        type_: str = None,
        code: list = None,
        status: list = None
    ) -> GetSMSDeliveryReportForBatchResponse:
        return self._sinch.configuration.transport.request(
            GetDeliveryReportForBatchEndpoint(
                sinch=self._sinch,
                request_data=GetSMSDeliveryReportForBatchRequest(
                    batch_id=batch_id,
                    type_=type_,
                    code=code,
                    status=status
                )
            )
        )

    def get_for_number(
        self,
        batch_id: str,
        recipient_number: str
    ) -> GetSMSDeliveryReportForNumberResponse:
        return self._sinch.configuration.transport.request(
            GetDeliveryReportForNumberEndpoint(
                sinch=self._sinch,
                request_data=GetSMSDeliveryReportForNumberRequest(
                    batch_id=batch_id,
                    recipient_number=recipient_number
                )
            )
        )


class SMSDeliveryReportsWithAsyncPagination(SMSDeliveryReports):
    async def list(
        self,
        page: int = 0,
        start_date: str = None,
        end_date: str = None,
        status: str = None,
        code: str = None,
        page_size: int = None,
        client_reference: str = None
    ) -> ListSMSDeliveryReportsResponse:
        return await AsyncIntBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListDeliveryReportsEndpoint(
                sinch=self._sinch,
                request_data=ListSMSDeliveryReportsRequest(
                    page=page,
                    page_size=page_size,
                    start_date=start_date,
                    end_date=end_date,
                    status=status,
                    code=code,
                    client_reference=client_reference
                )
            )
        )


class SMSInbounds:
    def __init__(self, sinch):
        self._sinch = sinch

    def list(
        self,
        page: int = 0,
        start_date: str = None,
        to: str = None,
        end_date: str = None,
        page_size: int = None,
        client_reference: str = None
    ) -> SinchListInboundMessagesResponse:
        return IntBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListInboundMessagesEndpoint(
                sinch=self._sinch,
                request_data=ListSMSInboundMessageRequest(
                    page=page,
                    page_size=page_size,
                    to=to,
                    end_date=end_date,
                    start_date=start_date,
                    client_reference=client_reference
                )
            )
        )

    def get(self, inbound_id: str) -> GetInboundMessagesResponse:
        return self._sinch.configuration.transport.request(
            GetInboundMessagesEndpoint(
                sinch=self._sinch,
                request_data=GetSMSInboundMessageRequest(
                    inbound_id=inbound_id
                )
            )
        )


class SMSInboundsWithAsyncPagination(SMSInbounds):
    async def list(
        self,
        page: int = 0,
        start_date: str = None,
        to: str = None,
        end_date: str = None,
        page_size: int = None,
        client_reference: str = None
    ) -> SinchListInboundMessagesResponse:
        return await AsyncIntBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListInboundMessagesEndpoint(
                sinch=self._sinch,
                request_data=ListSMSInboundMessageRequest(
                    page=page,
                    page_size=page_size,
                    to=to,
                    end_date=end_date,
                    start_date=start_date,
                    client_reference=client_reference
                )
            )
        )


class SMSBatches:
    def __init__(self, sinch):
        self._sinch = sinch

    def send(
        self,
        body: str,
        delivery_report: str,
        to: list,
        from_: str = None,
        parameters: dict = None,
        type_: str = None,
        send_at: str = None,
        expire_at: str = None,
        callback_url: str = None,
        client_reference: str = None,
        feedback_enabled: bool = None,
        flash_message: bool = None,
        truncate_concat: bool = None,
        max_number_of_message_parts: int = None,
        from_ton: int = None,
        from_npi: int = None
    ) -> SendSMSBatchResponse:
        return self._sinch.configuration.transport.request(
            SendBatchSMSEndpoint(
                sinch=self._sinch,
                request_data=SendBatchRequest(
                    to=to,
                    body=body,
                    from_=from_,
                    delivery_report=delivery_report,
                    feedback_enabled=feedback_enabled,
                    parameters=parameters,
                    type_=type_,
                    send_at=send_at,
                    expire_at=expire_at,
                    callback_url=callback_url,
                    client_reference=client_reference,
                    flash_message=flash_message,
                    truncate_concat=truncate_concat,
                    max_number_of_message_parts=max_number_of_message_parts,
                    from_npi=from_npi,
                    from_ton=from_ton
                )
            )
        )

    def list(
        self,
        page: int = 0,
        page_size: int = None,
        from_s: str = None,
        start_date: str = None,
        end_date: str = None,
        client_reference: str = None
    ) -> ListSMSBatchesResponse:
        return IntBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListSMSBatchesEndpoint(
                sinch=self._sinch,
                request_data=ListBatchesRequest(
                    page=page,
                    page_size=page_size,
                    from_s=from_s,
                    start_date=start_date,
                    end_date=end_date,
                    client_reference=client_reference
                )
            )
        )

    def get(self, batch_id: str) -> GetSMSBatchResponse:
        return self._sinch.configuration.transport.request(
            GetSMSEndpoint(
                sinch=self._sinch,
                request_data=GetBatchRequest(
                    batch_id=batch_id
                )
            )
        )

    def send_dry_run(
        self,
        to: str,
        body: str,
        per_recipient: bool = None,
        number_of_recipients: int = None,
        from_: str = None,
        type_: str = None,
        udh: str = None,
        delivery_report: str = None,
        send_at: str = None,
        expire_at: str = None,
        callback_url: str = None,
        flash_message: bool = None,
        parameters: dict = None,
        client_reference: str = None,
        max_number_of_message_parts: int = None
    ) -> SendSMSBatchDryRunResponse:
        return self._sinch.configuration.transport.request(
            SendBatchSMSDryRunEndpoint(
                sinch=self._sinch,
                request_data=BatchDryRunRequest(
                    per_recipient=per_recipient,
                    number_of_recipients=number_of_recipients,
                    to=to,
                    body=body,
                    from_=from_,
                    delivery_report=delivery_report,
                    type_=type_,
                    udh=udh,
                    send_at=send_at,
                    expire_at=expire_at,
                    callback_url=callback_url,
                    flash_message=flash_message,
                    parameters=parameters,
                    client_reference=client_reference,
                    max_number_of_message_parts=max_number_of_message_parts
                )
            )
        )

    def cancel(self, batch_id: str) -> CancelSMSBatchResponse:
        return self._sinch.configuration.transport.request(
            CancelBatchEndpoint(
                sinch=self._sinch,
                request_data=CancelBatchRequest(
                    batch_id=batch_id
                )
            )
        )

    def update(
        self,
        batch_id: str,
        to_add: list = None,
        to_remove: list = None,
        from_: str = None,
        body: str = None,
        delivery_report: str = None,
        send_at: str = None,
        expire_at: str = None,
        callback_url: str = None,
    ) -> UpdateSMSBatchResponse:
        return self._sinch.configuration.transport.request(
            UpdateBatchSMSEndpoint(
                sinch=self._sinch,
                request_data=UpdateBatchRequest(
                    batch_id=batch_id,
                    to_add=to_add,
                    to_remove=to_remove,
                    from_=from_,
                    body=body,
                    delivery_report=delivery_report,
                    send_at=send_at,
                    expire_at=expire_at,
                    callback_url=callback_url
                )
            )
        )

    def replace(
        self,
        batch_id: str,
        to: str,
        body: str,
        from_: str = None,
        type_: str = None,
        udh: str = None,
        delivery_report: str = None,
        send_at: str = None,
        expire_at: str = None,
        callback_url: str = None,
        flash_message: bool = None,
        parameters: dict = None,
        client_reference: str = None,
        max_number_of_message_parts: int = None
    ) -> ReplaceSMSBatchResponse:
        return self._sinch.configuration.transport.request(
            ReplaceBatchSMSEndpoint(
                sinch=self._sinch,
                request_data=ReplaceBatchRequest(
                    batch_id=batch_id,
                    to=to,
                    body=body,
                    from_=from_,
                    delivery_report=delivery_report,
                    type_=type_,
                    udh=udh,
                    send_at=send_at,
                    expire_at=expire_at,
                    callback_url=callback_url,
                    flash_message=flash_message,
                    client_reference=client_reference,
                    max_number_of_message_parts=max_number_of_message_parts,
                    parameters=parameters
                )
            )
        )

    def send_delivery_feedback(
        self,
        batch_id: str,
        recipients: list
    ) -> SendSMSDeliveryFeedbackResponse:
        return self._sinch.configuration.transport.request(
            SendDeliveryReportEndpoint(
                sinch=self._sinch,
                request_data=SendDeliveryFeedbackRequest(
                    batch_id=batch_id,
                    recipients=recipients
                )
            )
        )


class SMSBatchesWithAsyncPagination(SMSBatches):
    async def list(
        self,
        page: int = 0,
        page_size: int = None,
        from_s: str = None,
        start_date: str = None,
        end_date: str = None,
        client_reference: str = None
    ) -> ListSMSBatchesResponse:
        return await AsyncIntBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListSMSBatchesEndpoint(
                sinch=self._sinch,
                request_data=ListBatchesRequest(
                    page=page,
                    page_size=page_size,
                    from_s=from_s,
                    start_date=start_date,
                    end_date=end_date,
                    client_reference=client_reference
                )
            )
        )


class SMSGroups:
    def __init__(self, sinch):
        self._sinch = sinch

    def create(
        self,
        name: str,
        members: list = None,
        child_groups: list = None,
        auto_update: dict = None
    ) -> CreateSMSGroupResponse:
        return self._sinch.configuration.transport.request(
            CreateSMSGroupEndpoint(
                sinch=self._sinch,
                request_data=CreateSMSGroupRequest(
                    name=name,
                    members=members,
                    child_groups=child_groups,
                    auto_update=auto_update
                )
            )
        )

    def list(
        self,
        page=0,
        page_size=None
    ) -> SinchListSMSGroupResponse:
        return IntBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListSMSGroupEndpoint(
                sinch=self._sinch,
                request_data=ListSMSGroupRequest(
                    page=page,
                    page_size=page_size
                )
            )
        )

    def delete(
        self,
        group_id: str
    ) -> SinchDeleteSMSGroupResponse:
        return self._sinch.configuration.transport.request(
            DeleteSMSGroupEndpoint(
                sinch=self._sinch,
                request_data=DeleteSMSGroupRequest(
                    group_id=group_id
                )
            )
        )

    def get(
        self,
        group_id: str
    ) -> GetSMSGroupResponse:
        return self._sinch.configuration.transport.request(
            GetSMSGroupEndpoint(
                sinch=self._sinch,
                request_data=GetSMSGroupRequest(
                    group_id=group_id
                )
            )
        )

    def get_group_phone_numbers(
        self,
        group_id: str
    ) -> SinchGetSMSGroupPhoneNumbersResponse:
        return self._sinch.configuration.transport.request(
            GetSMSGroupPhoneNumbersEndpoint(
                sinch=self._sinch,
                request_data=GetSMSGroupPhoneNumbersRequest(
                    group_id=group_id
                )
            )
        )

    def update(
        self,
        group_id: str,
        name: str = None,
        add: list = None,
        remove: list = None,
        add_from_group: str = None,
        remove_from_group: str = None,
        auto_update: dict = None
    ) -> UpdateSMSGroupResponse:
        return self._sinch.configuration.transport.request(
            UpdateSMSGroupEndpoint(
                sinch=self._sinch,
                request_data=UpdateSMSGroupRequest(
                    group_id=group_id,
                    name=name,
                    add=add,
                    remove=remove,
                    add_from_group=add_from_group,
                    remove_from_group=remove_from_group,
                    auto_update=auto_update
                )
            )
        )

    def replace(
        self,
        group_id: str,
        members: list,
        name: str = None
    ) -> ReplaceSMSGroupResponse:
        return self._sinch.configuration.transport.request(
            ReplaceSMSGroupEndpoint(
                sinch=self._sinch,
                request_data=ReplaceSMSGroupPhoneNumbersRequest(
                    group_id=group_id,
                    members=members,
                    name=name
                )
            )
        )


class SMSGroupsWithAsyncPagination(SMSGroups):
    async def list(
        self,
        page=0,
        page_size=None
    ) -> SinchListSMSGroupResponse:
        return await AsyncIntBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListSMSGroupEndpoint(
                sinch=self._sinch,
                request_data=ListSMSGroupRequest(
                    page=page,
                    page_size=page_size
                )
            )
        )


class SMSBase:
    """
    Documentation for the SMS API: https://developers.sinch.com/docs/sms/
    """
    def __init__(self, sinch):
        self._sinch = sinch


class SMS(SMSBase):
    """
    Synchronous version of the SMS Domain
    """
    __doc__ += SMSBase.__doc__

    def __init__(self, sinch):
        super(SMS, self).__init__(sinch)
        self.groups = SMSGroups(self._sinch)
        self.batches = SMSBatches(self._sinch)
        self.inbounds = SMSInbounds(self._sinch)
        self.delivery_reports = SMSDeliveryReports(self._sinch)


class SMSAsync(SMSBase):
    """
    Asynchronous version of the SMS Domain
    """
    __doc__ += SMSBase.__doc__

    def __init__(self, sinch):
        super(SMSAsync, self).__init__(sinch)
        self.groups = SMSGroupsWithAsyncPagination(self._sinch)
        self.batches = SMSBatchesWithAsyncPagination(self._sinch)
        self.inbounds = SMSInboundsWithAsyncPagination(self._sinch)
        self.delivery_reports = SMSDeliveryReportsWithAsyncPagination(self._sinch)



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.groups.requests import GetSMSGroupRequest
from sinch.domains.sms.models.groups.responses import GetSMSGroupResponse


class GetSMSGroupEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/groups/{group_id}"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: GetSMSGroupRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id,
            group_id=self.request_data.group_id
        )

    def handle_response(self, response: HTTPResponse):
        super(GetSMSGroupEndpoint, self).handle_response(response)
        return GetSMSGroupResponse(
            id=response.body.get("id"),
            size=response.body.get("size"),
            created_at=response.body.get("created_at"),
            modified_at=response.body.get("modified_at"),
            name=response.body.get("name"),
            child_groups=response.body.get("child_groups"),
            auto_update=response.body.get("auto_update")
        )



import hashlib
import hmac
import base64
from datetime import datetime, timezone


class Signature:
    def __init__(
        self,
        sinch,
        http_method,
        request_data,
        request_uri,
        content_type=None,
        signature_timestamp=None
    ):
        self.sinch = sinch
        self.http_method = http_method
        self.content_type = content_type or 'application/json; charset=UTF-8'
        self.request_data = request_data
        self.signature_timestamp = signature_timestamp or datetime.now(timezone.utc).isoformat()
        self.request_uri = request_uri
        self.authorization_signature = None

    def get_http_headers_with_signature(self):
        if not self.authorization_signature:
            self.calculate()

        return {
            "Content-Type": self.content_type,
            "Authorization": (
                f"Application {self.sinch.configuration.application_key}:{self.authorization_signature}"
            ),
            "x-timestamp": self.signature_timestamp
        }

    def calculate(self):
        b64_decoded_application_secret = base64.b64decode(self.sinch.configuration.application_secret)
        if self.request_data:
            encoded_verification_request = hashlib.md5(self.request_data.encode())
            encoded_verification_request = base64.b64encode(encoded_verification_request.digest())

        else:
            encoded_verification_request = ''.encode()

        request_timestamp = "x-timestamp:" + self.signature_timestamp

        string_to_sign = (
            self.http_method + '\n'
            + encoded_verification_request.decode() + '\n'
            + self.content_type + '\n'
            + request_timestamp + '\n'
            + self.request_uri
        )

        self.authorization_signature = base64.b64encode(
            hmac.new(b64_decoded_application_secret, string_to_sign.encode(), hashlib.sha256).digest()
        ).decode()



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.delivery_reports.requests import GetSMSDeliveryReportForBatchRequest
from sinch.domains.sms.models.delivery_reports.responses import GetSMSDeliveryReportForBatchResponse


class GetDeliveryReportForBatchEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/batches/{batch_id}/delivery_report"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: GetSMSDeliveryReportForBatchRequest, sinch):
        super().__init__(request_data, sinch)

    def build_query_params(self):
        params = {}
        if self.request_data.type_:
            params["type"] = self.request_data.type_

        if self.request_data.status:
            params["status"] = self.request_data.status

        if self.request_data.code:
            params["code"] = list(map(str, self.request_data.code))

        return params

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id,
            batch_id=self.request_data.batch_id
        )

    def handle_response(self, response: HTTPResponse):
        super(GetDeliveryReportForBatchEndpoint, self).handle_response(response)
        return GetSMSDeliveryReportForBatchResponse(
            type=response.body.get("type"),
            batch_id=response.body.get("batch_id"),
            total_message_count=response.body.get("total_message_count"),
            statuses=response.body.get("statuses"),
            client_reference=response.body.get("client_reference")
        )



from dataclasses import dataclass
from typing import List

from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.numbers.models.regions import Region


@dataclass
class ListAvailableRegionsResponse(SinchBaseModel):
    available_regions: List[Region]



from enum import Enum


class HTTPMethods(Enum):
    GET = "GET"
    POST = "POST"
    DELETE = "DELETE"
    PATCH = "PATCH"
    PUT = "PUT"


class HTTPAuthentication(Enum):
    BASIC = "BASIC"
    OAUTH = "OAUTH"
    SIGNED = "SIGNED"
    SMS_TOKEN = "SMS_TOKEN"



from dataclasses import dataclass

from sinch.core.models.base_model import SinchBaseModel


@dataclass
class NumbersCallbackConfigurationResponse(SinchBaseModel):
    project_id: str
    hmac_secret: str


@dataclass
class GetNumbersCallbackConfigurationResponse(NumbersCallbackConfigurationResponse):
    pass


@dataclass
class UpdateNumbersCallbackConfigurationResponse(NumbersCallbackConfigurationResponse):
    pass



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class BatchRequest(SinchRequestBaseModel):
    def as_dict(self):
        payload = super(BatchRequest, self).as_dict()
        payload["to"] = payload.pop("to")
        if payload.get("from_"):
            payload["from"] = payload.pop("from_")
        return payload


@dataclass
class SendBatchRequest(BatchRequest):
    to: list
    from_: str
    body: str
    delivery_report: str
    parameters: dict
    send_at: str
    expire_at: str
    callback_url: str
    client_reference: str
    feedback_enabled: bool
    flash_message: bool
    truncate_concat: bool
    type_: str
    max_number_of_message_parts: int
    from_ton: int
    from_npi: int


@dataclass
class ListBatchesRequest(SinchRequestBaseModel):
    page_size: int
    from_s: str
    start_date: str
    end_date: str
    client_reference: str
    page: int = 0


@dataclass
class GetBatchRequest(SinchRequestBaseModel):
    batch_id: str


@dataclass
class CancelBatchRequest(SinchRequestBaseModel):
    batch_id: str


@dataclass
class BatchDryRunRequest(BatchRequest):
    per_recipient: bool
    number_of_recipients: int
    to: str
    from_: str
    body: str
    type_: str
    udh: str
    delivery_report: str
    parameters: dict
    send_at: str
    expire_at: str
    callback_url: str
    client_reference: str
    flash_message: bool
    max_number_of_message_parts: int


@dataclass
class UpdateBatchRequest(SinchRequestBaseModel):
    batch_id: str
    to_add: list
    to_remove: list
    from_: str
    body: str
    delivery_report: str
    send_at: str
    expire_at: str
    callback_url: str


@dataclass
class ReplaceBatchRequest(BatchRequest):
    batch_id: str
    to: str
    from_: str
    body: str
    delivery_report: str
    parameters: dict
    send_at: str
    expire_at: str
    type_: str
    callback_url: str
    client_reference: str
    flash_message: bool
    max_number_of_message_parts: int
    udh: str


@dataclass
class SendDeliveryFeedbackRequest(SinchRequestBaseModel):
    batch_id: str
    recipients: list



from dataclasses import dataclass
from typing import Literal
from sinch.core.models.base_model import SinchRequestBaseModel
from sinch.domains.voice.models import Destination, ConferenceDTMFOptions


@dataclass
class TextToSpeechVoiceCalloutRequest(SinchRequestBaseModel):
    destination: Destination
    cli: str
    dtmf: str
    domain: Literal["pstn", "mxp"]
    custom: str
    locale: str
    text: str
    prompts: str
    enableAce: bool
    enableDice: bool
    enablePie: bool


@dataclass
class ConferenceVoiceCalloutRequest(SinchRequestBaseModel):
    destination: Destination
    conferenceId: str
    cli: str
    conferenceDtmfOptions: ConferenceDTMFOptions
    dtmf: str
    conference: str
    maxDuration: int
    enableAce: bool
    enableDice: bool
    enablePie: bool
    locale: str
    greeting: str
    mohClass: str
    custom: str
    domain: Literal["pstn", "mxp"]


@dataclass
class CustomVoiceCalloutRequest(SinchRequestBaseModel):
    cli: str
    destination: Destination
    dtmf: str
    custom: str
    maxDuration: int
    ice: str
    ace: str
    pie: str



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.numbers.endpoints.numbers_endpoint import NumbersEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.numbers.models.active import ActiveNumber
from sinch.domains.numbers.models.active.requests import ListActiveNumbersRequest
from sinch.domains.numbers.models.active.responses import ListActiveNumbersResponse


class ListActiveNumbersEndpoint(NumbersEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/activeNumbers"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: ListActiveNumbersRequest):
        super(ListActiveNumbersEndpoint, self).__init__(project_id, request_data)
        self.project_id = project_id
        self.request_data = request_data

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.numbers_origin,
            project_id=self.project_id
        )

    def build_query_params(self) -> dict:
        params = {
            "regionCode": self.request_data.region_code,
            "type": self.request_data.number_type,
        }

        if self.request_data.capabilities:
            params["capabilities"] = self.request_data.capabilities

        if self.request_data.number_pattern:
            params["numberPattern.pattern"] = self.request_data.number_pattern

        if self.request_data.number_search_pattern:
            params["numberPattern.searchPattern"] = self.request_data.capabilities

        if self.request_data.page_size:
            params["pageSize"] = self.request_data.page_size

        if self.request_data.page_token:
            params["pageToken"] = self.request_data.page_token

        return params

    def handle_response(self, response: HTTPResponse) -> ListActiveNumbersResponse:
        super(ListActiveNumbersEndpoint, self).handle_response(response)
        return ListActiveNumbersResponse(
            [
                ActiveNumber(
                    phone_number=number["phoneNumber"],
                    project_id=number["projectId"],
                    display_name=number["displayName"],
                    region_code=number["regionCode"],
                    type=number["type"],
                    capability=number["capability"],
                    money=number["money"],
                    payment_interval_months=number["paymentIntervalMonths"],
                    next_charge_date=number["nextChargeDate"],
                    expire_at=number["expireAt"],
                    sms_configuration=number["smsConfiguration"],
                    voice_configuration=number["voiceConfiguration"]
                ) for number in response.body["activeNumbers"]
            ],
            next_page_token=response.body["nextPageToken"]
        )



from enum import Enum
from abc import ABC, abstractmethod
from sinch.domains.authentication.models.authentication import OAuthToken
from sinch.domains.authentication.endpoints.oauth import OAuthEndpoint
from sinch.core.exceptions import ValidationException


class TokenState(Enum):
    VALID = "VALID"
    INVALID = "INVALID"
    EXPIRED = "EXPIRED"


class TokenManagerBase(ABC):
    def __init__(self, sinch):
        self.sinch = sinch
        self.token = None
        self.token_state = TokenState.INVALID

    @abstractmethod
    def get_auth_token(self) -> OAuthToken:
        pass

    def invalidate_expired_token(self):
        self.token = None
        self.token_state = TokenState.EXPIRED

    def handle_invalid_token(self, http_response):
        if http_response.headers.get("www-authenticate") and "expired" in http_response.headers["www-authenticate"]:
            self.invalidate_expired_token()

    def set_auth_token(self, token) -> None:
        try:
            self.token = OAuthToken(**token)
            self.token_state = TokenState.VALID
        except TypeError:
            raise ValidationException(
                "Invalid authentication token structure",
                is_from_server=False,
                response=None
            )


class TokenManager(TokenManagerBase):
    def get_auth_token(self) -> OAuthToken:
        if self.token:
            return self.token

        self.token = self.sinch.configuration.transport.request(OAuthEndpoint())
        self.token_state = TokenState.VALID
        return self.token


class TokenManagerAsync(TokenManagerBase):
    async def get_auth_token(self) -> OAuthToken:
        if self.token:
            return self.token

        self.token = await self.sinch.configuration.transport.request(OAuthEndpoint())
        self.token_state = TokenState.VALID
        return self.token



from typing import Union, List
from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel
from sinch.domains.voice.models.svaml.actions.actions import Action
from sinch.domains.voice.models.svaml.instructions.instructions import Instruction


@dataclass
class GetVoiceCallRequest(SinchRequestBaseModel):
    call_id: str


@dataclass
class UpdateVoiceCallRequest(SinchRequestBaseModel):
    call_id: str
    instructions: Union[list, List[Instruction]]
    action: Action


@dataclass
class ManageVoiceCallRequest(SinchRequestBaseModel):
    call_id: str
    call_leg: str
    instructions: Union[list, List[Instruction]]
    action: Action



import requests
from sinch.core.ports.http_transport import HTTPTransport, HttpRequest
from sinch.core.endpoint import HTTPEndpoint
from sinch.core.models.http_response import HTTPResponse


class HTTPTransportRequests(HTTPTransport):
    def __init__(self, sinch):
        super().__init__(sinch)
        self.http_session = requests.Session()

    def request(self, endpoint: HTTPEndpoint) -> HTTPResponse:
        request_data: HttpRequest = self.prepare_request(endpoint)
        request_data: HttpRequest = self.authenticate(endpoint, request_data)

        self.sinch.configuration.logger.debug(
            f"Sync HTTP {request_data.http_method} call with headers:"
            f" {request_data.headers} and body: {request_data.request_body} to URL: {request_data.url}"
        )

        response = self.http_session.request(
            method=request_data.http_method,
            url=request_data.url,
            data=request_data.request_body,
            auth=request_data.auth,
            headers=request_data.headers,
            timeout=self.sinch.configuration.connection_timeout,
            params=request_data.query_params
        )

        response_body = self.deserialize_json_response(response)

        self.sinch.configuration.logger.debug(
            f"Sync HTTP {response.status_code} response with headers: {response.headers}"
            f"and body: {response_body} from URL: {request_data.url}"
        )

        return self.handle_response(
            endpoint=endpoint,
            http_response=HTTPResponse(
                status_code=response.status_code,
                body=response_body,
                headers=response.headers
            )
        )



from dataclasses import dataclass
from typing import List

from sinch.core.models.base_model import SinchBaseModel
from sinch.domains.numbers.models import Number


@dataclass
class ListAvailableNumbersResponse(SinchBaseModel):
    available_numbers: List[Number]


@dataclass
class ActivateNumberResponse(SinchBaseModel):
    phone_number: str
    region_code: str
    type: str
    capability: list


@dataclass
class RentAnyNumberResponse(SinchBaseModel):
    phone_number: str
    project_id: str
    region_code: str
    type: str
    capability: list
    money: dict
    payment_interval_months: int
    next_charge_date: str
    expire_at: str
    sms_configuration: object
    voice_configuration: object
    callback_url: str
    capability: tuple


@dataclass
class CheckNumberAvailabilityResponse(Number):
    pass



from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel


@dataclass
class DeliveryReport(SinchBaseModel):
    at: str
    batch_id: str
    code: int
    recipient: str
    status: str
    applied_originator: str
    client_reference: str
    encoding: str
    number_of_message_parts: int
    operator: str
    operator_status_at: str
    type: str



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.voice.endpoints.voice_endpoint import VoiceEndpoint
from sinch.domains.voice.models import ApplicationNumber
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.voice.models.applications.responses import GetNumbersVoiceApplicationResponse


class GetVoiceNumbersEndpoint(VoiceEndpoint):
    ENDPOINT_URL = "{origin}/v1/configuration/numbers"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.SIGNED.value

    def __init__(self):
        pass

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.voice_applications_origin
        )

    def handle_response(self, response: HTTPResponse) -> GetNumbersVoiceApplicationResponse:
        super().handle_response(response)
        return GetNumbersVoiceApplicationResponse(
            numbers=[
                ApplicationNumber(
                    number=number.get("number"),
                    capability=number.get("capability"),
                    applicationkey=number.get("applicationkey")
                ) for number in response.body["numbers"]
            ]
        )



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.batches.responses import SendSMSBatchDryRunResponse
from sinch.domains.sms.models.batches.requests import BatchDryRunRequest


class SendBatchSMSDryRunEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/batches/dry_run"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: BatchDryRunRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id
        )

    def build_query_params(self):
        return {
            "per_recipient": str(self.request_data.per_recipient).lower(),
            "number_of_recipients": self.request_data.number_of_recipients
        }

    def request_body(self):
        self.request_data.per_recipient = None
        self.request_data.number_of_recipients = None
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse):
        super(SendBatchSMSDryRunEndpoint, self).handle_response(response)
        return SendSMSBatchDryRunResponse(
            number_of_messages=response.body.get("number_of_messages"),
            number_of_recipients=response.body.get("number_of_recipients"),
            per_recipient=response.body.get("per_recipient")
        )



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.message.responses import GetConversationMessageResponse
from sinch.domains.conversation.models.message.requests import GetConversationMessageRequest


class GetConversationMessageEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/messages/{message_id}"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: GetConversationMessageRequest):
        super(GetConversationMessageEndpoint, self).__init__(project_id, request_data)
        self.request_data = request_data
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id,
            message_id=self.request_data.message_id
        )

    def build_query_params(self):
        if self.request_data.messages_source:
            return {
                "messages_source": self.request_data.messages_source
            }

    def handle_response(self, response: HTTPResponse) -> GetConversationMessageResponse:
        return GetConversationMessageResponse(
            id=response.body["id"],
            direction=response.body["direction"],
            channel_identity=response.body["channel_identity"],
            app_message=response.body["app_message"],
            conversation_id=response.body["conversation_id"],
            contact_id=response.body["contact_id"],
            metadata=response.body["metadata"],
            accept_time=response.body["accept_time"],
            sender_id=response.body["sender_id"],
            processing_mode=response.body["processing_mode"],
        )



import aiohttp
from abc import ABC, abstractmethod
from platform import python_version
from sinch.core.endpoint import HTTPEndpoint
from sinch.core.signature import Signature
from sinch.core.models.http_request import HttpRequest
from sinch.core.models.http_response import HTTPResponse
from sinch.core.exceptions import ValidationException, SinchException
from sinch.core.enums import HTTPAuthentication
from sinch.core.token_manager import TokenState
from sinch import __version__ as sdk_version


class HTTPTransport(ABC):
    def __init__(self, sinch):
        self.sinch = sinch

    @abstractmethod
    def request(self, endpoint: HTTPEndpoint) -> HTTPResponse:
        pass

    def authenticate(self, endpoint, request_data):
        if endpoint.HTTP_AUTHENTICATION in (HTTPAuthentication.BASIC.value, HTTPAuthentication.OAUTH.value):
            if (
                not self.sinch.configuration.key_id
                or not self.sinch.configuration.key_secret
                or not self.sinch.configuration.project_id
            ):
                raise ValidationException(
                    message=(
                        "key_id, key_secret and project_id are required by this API. "
                        "Those credentials can be obtained from Sinch portal."
                    ),
                    is_from_server=False,
                    response=None
                )

        if endpoint.HTTP_AUTHENTICATION == HTTPAuthentication.BASIC.value:
            request_data.auth = (self.sinch.configuration.key_id, self.sinch.configuration.key_secret)
        else:
            request_data.auth = None

        if endpoint.HTTP_AUTHENTICATION == HTTPAuthentication.OAUTH.value:
            token = self.sinch.authentication.get_auth_token().access_token
            request_data.headers.update({
                "Authorization": f"Bearer {token}",
                "Content-Type": "application/json"
            })
        elif endpoint.HTTP_AUTHENTICATION == HTTPAuthentication.SIGNED.value:
            if not self.sinch.configuration.application_key or not self.sinch.configuration.application_secret:
                raise ValidationException(
                    message=(
                        "application key and application secret are required by this API. "
                        "Those credentials can be obtained from Sinch portal."
                    ),
                    is_from_server=False,
                    response=None
                )
            signature = Signature(
                self.sinch,
                endpoint.HTTP_METHOD,
                request_data.request_body,
                endpoint.get_url_without_origin(self.sinch)
            )
            request_data.headers = signature.get_http_headers_with_signature()
        elif endpoint.HTTP_AUTHENTICATION == HTTPAuthentication.SMS_TOKEN.value:
            if not self.sinch.configuration.sms_api_token or not self.sinch.configuration.service_plan_id:
                raise ValidationException(
                    message=(
                        "sms_api_token and service_plan_id are required by this API. "
                        "Those credentials can be obtained from Sinch portal."
                    ),
                    is_from_server=False,
                    response=None
                )
            request_data.headers.update({
                "Authorization": f"Bearer {self.sinch.configuration.sms_api_token}",
                "Content-Type": "application/json"
            })

        return request_data

    def prepare_request(self, endpoint: HTTPEndpoint) -> HttpRequest:
        protocol = "http://" if self.sinch.configuration.disable_https else "https://"
        url_query_params = endpoint.build_query_params()

        return HttpRequest(
            headers={
                "User-Agent": f"sinch-sdk/{sdk_version} (Python/{python_version()};"
                              f" {self.__class__.__name__};)"
            },
            protocol=protocol,
            url=protocol + endpoint.build_url(self.sinch),
            http_method=endpoint.HTTP_METHOD,
            request_body=endpoint.request_body(),
            query_params=url_query_params,
            auth=()
        )

    @staticmethod
    def deserialize_json_response(response):
        if response.content:
            try:
                response_body = response.json()
            except ValueError as err:
                raise SinchException(
                    message=(
                        "Error while parsing json response.",
                        err.msg
                    ),
                    is_from_server=True,
                    response=response
                )
        else:
            response_body = {}

        return response_body

    def handle_response(self, endpoint: HTTPEndpoint, http_response: HTTPResponse):
        if http_response.status_code == 401 and endpoint.HTTP_AUTHENTICATION == HTTPAuthentication.OAUTH.value:
            self.sinch.configuration.token_manager.handle_invalid_token(http_response)
            if self.sinch.configuration.token_manager.token_state == TokenState.EXPIRED:
                return self.request(endpoint=endpoint)

        return endpoint.handle_response(http_response)


class AsyncHTTPTransport(HTTPTransport):
    async def authenticate(self, endpoint, request_data):
        if endpoint.HTTP_AUTHENTICATION == HTTPAuthentication.BASIC.value:
            request_data.auth = aiohttp.BasicAuth(
                self.sinch.configuration.key_id,
                self.sinch.configuration.key_secret
            )
        else:
            request_data.auth = None

        if endpoint.HTTP_AUTHENTICATION == HTTPAuthentication.OAUTH.value:
            token_response = await self.sinch.authentication.get_auth_token()
            request_data.headers = {
                "Authorization": f"Bearer {token_response.access_token}",
                "Content-Type": "application/json"
            }

        return request_data

    async def handle_response(self, endpoint: HTTPEndpoint, http_response: HTTPResponse):
        if http_response.status_code == 401 and endpoint.HTTP_AUTHENTICATION == HTTPAuthentication.OAUTH.value:
            self.sinch.configuration.token_manager.handle_invalid_token(http_response)
            if self.sinch.configuration.token_manager.token_state == TokenState.EXPIRED:
                return await self.request(endpoint=endpoint)

        return endpoint.handle_response(http_response)



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.numbers.endpoints.numbers_endpoint import NumbersEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.numbers.models.callbacks.responses import UpdateNumbersCallbackConfigurationResponse
from sinch.domains.numbers.models.callbacks.requests import UpdateNumbersCallbackConfigurationRequest


class UpdateNumbersCallbackConfigurationEndpoint(NumbersEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/callbackConfiguration"
    HTTP_METHOD = HTTPMethods.PATCH.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: UpdateNumbersCallbackConfigurationRequest):
        super().__init__(project_id, request_data)
        self.project_id = project_id
        self.request_data = request_data

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.numbers_origin,
            project_id=self.project_id
        )

    def request_body(self):
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse) -> UpdateNumbersCallbackConfigurationResponse:
        super().handle_response(response)
        return UpdateNumbersCallbackConfigurationResponse(
            project_id=response.body['projectId'],
            hmac_secret=response.body['hmacSecret']
        )



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.opt_in_opt_out.requests import RegisterConversationOptInRequest
from sinch.domains.conversation.models.opt_in_opt_out.responses import RegisterConversationOptInResponse


class RegisterOptInEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/optins:register"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: RegisterConversationOptInRequest):
        super(RegisterOptInEndpoint, self).__init__(project_id, request_data)
        self.request_data = request_data
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id
        )

    def build_query_params(self):
        if self.request_data.request_id:
            return {
                "request_id": self.request_data.request_id
            }

    def request_body(self):
        self.request_data.request_id = None
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse) -> RegisterConversationOptInResponse:
        super(RegisterOptInEndpoint, self).handle_response(response)
        return RegisterConversationOptInResponse(
            response.body["request_id"],
            response.body["opt_in"]
        )



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.batches.responses import UpdateSMSBatchResponse
from sinch.domains.sms.models.batches.requests import UpdateBatchRequest


class UpdateBatchSMSEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/batches/{batch_id}"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: UpdateBatchRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id,
            batch_id=self.request_data.batch_id
        )

    def request_body(self):
        self.request_data.batch_id = None
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse):
        super(UpdateBatchSMSEndpoint, self).handle_response(response)
        return UpdateSMSBatchResponse(
            id=response.body.get("id"),
            to=response.body.get("to"),
            from_=response.body.get("from"),
            body=response.body.get("body"),
            delivery_report=response.body.get("delivery_report"),
            cancelled=response.body.get("cancelled"),
            type=response.body.get("type"),
            campaign_id=response.body.get("campaign_id"),
            created_at=response.body.get("created_at"),
            modified_at=response.body.get("modified_at"),
            send_at=response.body.get("send_at"),
            expire_at=response.body.get("expire_at"),
            callback_url=response.body.get("callback_url")
        )



from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel


@dataclass
class ConversationTemplate(SinchBaseModel):
    id: str
    description: str
    default_translation: str
    create_time: str
    translations: list
    update_time: str
    channel: str



from dataclasses import dataclass
from sinch.core.models.base_model import SinchBaseModel


@dataclass
class SMSGroup(SinchBaseModel):
    id: str
    size: int
    created_at: str
    modified_at: str
    name: str
    child_groups: list
    auto_update: dict



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.numbers.endpoints.numbers_endpoint import NumbersEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.numbers.models import Number

from sinch.domains.numbers.models.available.requests import ListAvailableNumbersRequest
from sinch.domains.numbers.models.available.responses import ListAvailableNumbersResponse


class AvailableNumbersEndpoint(NumbersEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/availableNumbers"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: ListAvailableNumbersRequest):
        super(AvailableNumbersEndpoint, self).__init__(project_id, request_data)
        self.project_id = project_id
        self.request_data = request_data

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.numbers_origin,
            project_id=self.project_id
        )

    def build_query_params(self) -> dict:
        query_params = {
            "regionCode": self.request_data.region_code,
            "type": self.request_data.number_type
        }

        if self.request_data.page_size:
            query_params["size"] = self.request_data.page_size

        if self.request_data.capabilities:
            query_params["capabilities"] = self.request_data.capabilities

        if self.request_data.number_pattern:
            query_params["numberPattern.pattern"] = self.request_data.number_pattern

        if self.request_data.number_search_pattern:
            query_params["numberPattern.searchPattern"] = self.request_data.number_search_pattern

        return query_params

    def handle_response(self, response: HTTPResponse) -> ListAvailableNumbersResponse:
        super(AvailableNumbersEndpoint, self).handle_response(response)
        return ListAvailableNumbersResponse(
            [
                Number(
                    phone_number=number["phoneNumber"],
                    region_code=number["regionCode"],
                    type=number["type"],
                    capability=number["capability"],
                    setup_price=number["setupPrice"],
                    monthly_price=number["monthlyPrice"],
                    payment_interval_months=number["paymentIntervalMonths"],
                    supporting_documentation_required=number["supportingDocumentationRequired"]
                ) for number in response.body["availableNumbers"]
            ]
        )



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class ConversationRequest(SinchRequestBaseModel):
    app_id: str
    contact_id: str
    active: bool
    active_channel: str
    app_id: str
    contact_id: str
    metadata: str
    conversation_metadata: dict


@dataclass
class CreateConversationRequest(ConversationRequest):
    id: str


@dataclass
class ListConversationsRequest(SinchRequestBaseModel):
    app_id: str
    contact_id: str
    only_active: bool
    page_size: int
    page_token: str


@dataclass
class GetConversationRequest(SinchRequestBaseModel):
    conversation_id: str


@dataclass
class DeleteConversationRequest(SinchRequestBaseModel):
    conversation_id: str


@dataclass
class UpdateConversationRequest(ConversationRequest):
    update_mask: str
    metadata_update_strategy: str
    conversation_id: str


@dataclass
class StopConversationRequest(SinchRequestBaseModel):
    conversation_id: str


@dataclass
class InjectMessageToConversationRequest(SinchRequestBaseModel):
    conversation_id: str
    accept_time: str
    app_message: dict
    channel_identity: dict
    contact_id: str
    contact_message: dict
    direction: str
    metadata: str



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.numbers.endpoints.numbers_endpoint import NumbersEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.numbers.models.available.requests import ActivateNumberRequest
from sinch.domains.numbers.models.available.responses import ActivateNumberResponse


class ActivateNumberEndpoint(NumbersEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/availableNumbers/{phone_number}:rent"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: ActivateNumberRequest):
        super(ActivateNumberEndpoint, self).__init__(project_id, request_data)
        self.project_id = project_id
        self.request_data = request_data

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.numbers_origin,
            project_id=self.project_id,
            phone_number=self.request_data.phone_number
        )

    def handle_response(self, response: HTTPResponse) -> ActivateNumberResponse:
        super(ActivateNumberEndpoint, self).handle_response(response)
        return ActivateNumberResponse(
            phone_number=response.body["phoneNumber"],
            region_code=response.body["regionCode"],
            type=response.body["type"],
            capability=response.body["capability"]
        )



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.webhook.responses import CreateWebhookResponse
from sinch.domains.conversation.models.webhook.requests import CreateConversationWebhookRequest


class CreateWebhookEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/webhooks"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: CreateConversationWebhookRequest):
        super(CreateWebhookEndpoint, self).__init__(project_id, request_data)
        self.request_data = request_data
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id
        )

    def request_body(self):
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse) -> CreateWebhookResponse:
        super(CreateWebhookEndpoint, self).handle_response(response)
        return CreateWebhookResponse(
            id=response.body["id"],
            app_id=response.body["app_id"],
            target=response.body["target"],
            target_type=response.body["target_type"],
            secret=response.body["secret"],
            triggers=response.body["triggers"],
            client_credentials=response.body["client_credentials"]
        )



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.conversation.endpoints.conversation_endpoint import ConversationEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.conversation.models.contact.responses import CreateConversationContactResponse
from sinch.domains.conversation.models.contact.requests import CreateConversationContactRequest


class CreateConversationContactEndpoint(ConversationEndpoint):
    ENDPOINT_URL = "{origin}/v1/projects/{project_id}/contacts"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, project_id: str, request_data: CreateConversationContactRequest):
        super(CreateConversationContactEndpoint, self).__init__(project_id, request_data)
        self.request_data = request_data
        self.project_id = project_id

    def build_url(self, sinch):
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.conversation_origin,
            project_id=self.project_id
        )

    def request_body(self):
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse) -> CreateConversationContactResponse:
        super(CreateConversationContactEndpoint, self).handle_response(response)
        return CreateConversationContactResponse(
            id=response.body["id"],
            channel_identities=response.body["channel_identities"],
            channel_priority=response.body["channel_priority"],
            display_name=response.body["display_name"],
            email=response.body["email"],
            external_id=response.body["external_id"],
            metadata=response.body["metadata"],
            language=response.body["language"]
        )



from typing import List

from sinch.core.pagination import TokenBasedPaginator, AsyncTokenBasedPaginator

from sinch.domains.conversation.models import (
    SinchConversationChannelIdentities,
    SinchConversationRecipient,
    ConversationChannel
)

from sinch.domains.conversation.models.app.requests import (
    CreateConversationAppRequest,
    DeleteConversationAppRequest,
    GetConversationAppRequest,
    UpdateConversationAppRequest
)

from sinch.domains.conversation.models.app.responses import (
    CreateConversationAppResponse,
    DeleteConversationAppResponse,
    ListConversationAppsResponse,
    GetConversationAppResponse,
    UpdateConversationAppResponse
)

from sinch.domains.conversation.models.contact.requests import (
    CreateConversationContactRequest,
    UpdateConversationContactRequest,
    ListConversationContactRequest,
    DeleteConversationContactRequest,
    GetConversationContactRequest,
    MergeConversationContactsRequest,
    GetConversationChannelProfileRequest
)

from sinch.domains.conversation.models.contact.responses import (
    UpdateConversationContactResponse,
    ListConversationContactsResponse,
    DeleteConversationContactResponse,
    MergeConversationContactsResponse,
    CreateConversationContactResponse,
    GetConversationContactResponse,
    GetConversationChannelProfileResponse
)

from sinch.domains.conversation.models.message.requests import (
    SendConversationMessageRequest,
    ListConversationMessagesRequest,
    DeleteConversationMessageRequest,
    GetConversationMessageRequest
)

from sinch.domains.conversation.models.message.responses import (
    SendConversationMessageResponse,
    ListConversationMessagesResponse,
    GetConversationMessageResponse,
    DeleteConversationMessageResponse
)

from sinch.domains.conversation.models.conversation.requests import (
    CreateConversationRequest,
    ListConversationsRequest,
    GetConversationRequest,
    DeleteConversationRequest,
    UpdateConversationRequest,
    StopConversationRequest,
    InjectMessageToConversationRequest
)

from sinch.domains.conversation.models.conversation.responses import (
    SinchCreateConversationResponse,
    SinchUpdateConversationResponse,
    SinchGetConversationResponse,
    SinchDeleteConversationResponse,
    SinchListConversationsResponse,
    SinchStopConversationResponse,
    SinchInjectMessageResponse
)

from sinch.domains.conversation.models.webhook.requests import (
    CreateConversationWebhookRequest,
    GetConversationWebhookRequest,
    DeleteConversationWebhookRequest,
    UpdateConversationWebhookRequest,
    ListConversationWebhookRequest
)

from sinch.domains.conversation.models.webhook.responses import (
    CreateWebhookResponse,
    GetWebhookResponse,
    SinchListWebhooksResponse,
    SinchDeleteWebhookResponse,
    UpdateWebhookResponse
)

from sinch.domains.conversation.models.templates.requests import (
    CreateConversationTemplateRequest,
    GetConversationTemplateRequest,
    DeleteConversationTemplateRequest,
    UpdateConversationTemplateRequest
)

from sinch.domains.conversation.models.templates.responses import (
    CreateConversationTemplateResponse,
    UpdateConversationTemplateResponse,
    DeleteConversationTemplateResponse,
    ListConversationTemplatesResponse,
    GetConversationTemplateResponse
)

from sinch.domains.conversation.models.event.requests import SendConversationEventRequest
from sinch.domains.conversation.models.event.responses import SendConversationEventResponse

from sinch.domains.conversation.models.opt_in_opt_out.requests import RegisterConversationOptInRequest
from sinch.domains.conversation.models.opt_in_opt_out.responses import RegisterConversationOptInResponse

from sinch.domains.conversation.models.opt_in_opt_out.requests import RegisterConversationOptOutRequest
from sinch.domains.conversation.models.opt_in_opt_out.responses import RegisterConversationOptOutResponse

from sinch.domains.conversation.models.capability.requests import QueryConversationCapabilityRequest
from sinch.domains.conversation.models.capability.responses import QueryConversationCapabilityResponse

from sinch.domains.conversation.models.transcoding.requests import TranscodeConversationMessageRequest
from sinch.domains.conversation.models.transcoding.responses import TranscodeConversationMessageResponse

from sinch.domains.conversation.endpoints.message.send_message import SendConversationMessageEndpoint
from sinch.domains.conversation.endpoints.message.list_message import ListConversationMessagesEndpoint
from sinch.domains.conversation.endpoints.message.get_message import GetConversationMessageEndpoint
from sinch.domains.conversation.endpoints.message.delete_message import DeleteConversationMessageEndpoint
from sinch.domains.conversation.endpoints.contact.list_contact import ListContactsEndpoint
from sinch.domains.conversation.endpoints.contact.create_contact import CreateConversationContactEndpoint
from sinch.domains.conversation.endpoints.contact.get_contact import GetContactEndpoint
from sinch.domains.conversation.endpoints.contact.delete_contact import DeleteContactEndpoint
from sinch.domains.conversation.endpoints.contact.update_contact import UpdateConversationContactEndpoint
from sinch.domains.conversation.endpoints.contact.merge_contacts import MergeConversationContactsEndpoint
from sinch.domains.conversation.endpoints.contact.get_channel_profile import GetChannelProfileEndpoint
from sinch.domains.conversation.endpoints.app.create_app import CreateConversationAppEndpoint
from sinch.domains.conversation.endpoints.app.delete_app import DeleteConversationAppEndpoint
from sinch.domains.conversation.endpoints.app.list_apps import ListAppsEndpoint
from sinch.domains.conversation.endpoints.app.get_app import GetAppEndpoint
from sinch.domains.conversation.endpoints.app.update_app import UpdateConversationAppEndpoint
from sinch.domains.conversation.endpoints.conversation.create_conversation import CreateConversationEndpoint
from sinch.domains.conversation.endpoints.conversation.list_conversations import ListConversationsEndpoint
from sinch.domains.conversation.endpoints.conversation.get_conversation import GetConversationEndpoint
from sinch.domains.conversation.endpoints.conversation.delete_conversation import DeleteConversationEndpoint
from sinch.domains.conversation.endpoints.conversation.update_conversation import UpdateConversationEndpoint
from sinch.domains.conversation.endpoints.conversation.stop_conversation import StopConversationEndpoint
from sinch.domains.conversation.endpoints.conversation.inject_message_to_conversation import (
    InjectMessageToConversationEndpoint
)
from sinch.domains.conversation.endpoints.webhooks.create_webhook import CreateWebhookEndpoint
from sinch.domains.conversation.endpoints.webhooks.list_webhooks import ListWebhooksEndpoint
from sinch.domains.conversation.endpoints.webhooks.get_webhook import GetWebhookEndpoint
from sinch.domains.conversation.endpoints.webhooks.delete_webhook import DeleteWebhookEndpoint
from sinch.domains.conversation.endpoints.webhooks.update_webhook import UpdateWebhookEndpoint
from sinch.domains.conversation.endpoints.templates.create_template import CreateTemplateEndpoint
from sinch.domains.conversation.endpoints.templates.list_templates import ListTemplatesEndpoint
from sinch.domains.conversation.endpoints.templates.get_template import GetTemplatesEndpoint
from sinch.domains.conversation.endpoints.templates.delete_template import DeleteTemplateEndpoint
from sinch.domains.conversation.endpoints.templates.update_template import UpdateTemplateEndpoint
from sinch.domains.conversation.endpoints.events import SendEventEndpoint
from sinch.domains.conversation.endpoints.transcode import TranscodeMessageEndpoint
from sinch.domains.conversation.endpoints.opt_in import RegisterOptInEndpoint
from sinch.domains.conversation.endpoints.opt_out import RegisterOptOutEndpoint
from sinch.domains.conversation.endpoints.capability import CapabilityQueryEndpoint


class ConversationMessage:
    def __init__(self, sinch):
        self._sinch = sinch

    def send(
        self,
        app_id: str,
        recipient: dict,
        message: dict,
        callback_url: str = None,
        channel_priority_order: list = None,
        channel_properties: dict = None,
        message_metadata: str = None,
        conversation_metadata: dict = None,
        queue: str = None,
        ttl: str = None,
        processing_strategy: str = None
    ) -> SendConversationMessageResponse:
        return self._sinch.configuration.transport.request(
            SendConversationMessageEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=SendConversationMessageRequest(
                    app_id=app_id,
                    recipient=recipient,
                    message=message,
                    callback_url=callback_url,
                    channel_priority_order=channel_priority_order,
                    channel_properties=channel_properties,
                    message_metadata=message_metadata,
                    conversation_metadata=conversation_metadata,
                    queue=queue,
                    ttl=ttl,
                    processing_strategy=processing_strategy
                )
            )
        )

    def get(
        self,
        message_id: str,
        messages_source: str = None
    ) -> GetConversationMessageResponse:
        return self._sinch.configuration.transport.request(
            GetConversationMessageEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=GetConversationMessageRequest(
                    message_id=message_id,
                    messages_source=messages_source
                )
            )
        )

    def delete(
        self,
        message_id: str,
        messages_source: str = None
    ) -> DeleteConversationMessageResponse:
        return self._sinch.configuration.transport.request(
            DeleteConversationMessageEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=DeleteConversationMessageRequest(
                    message_id=message_id,
                    messages_source=messages_source
                )
            )
        )

    def list(
        self,
        conversation_id: str = None,
        contact_id: str = None,
        app_id: str = None,
        page_size: int = None,
        page_token: str = None,
        view: str = None,
        messages_source: str = None,
        only_recipient_originated: bool = None
    ) -> ListConversationMessagesResponse:
        return TokenBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListConversationMessagesEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ListConversationMessagesRequest(
                    contact_id=contact_id,
                    conversation_id=conversation_id,
                    app_id=app_id,
                    page_size=page_size,
                    page_token=page_token,
                    view=view,
                    messages_source=messages_source,
                    only_recipient_originated=only_recipient_originated
                )
            )
        )


class ConversationMessageWithAsyncPagination(ConversationMessage):
    async def list(
        self,
        conversation_id: str = None,
        contact_id: str = None,
        app_id: str = None,
        page_size: int = None,
        page_token: str = None,
        view: str = None,
        messages_source: str = None,
        only_recipient_originated: bool = None
    ) -> ListConversationMessagesResponse:
        return await AsyncTokenBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListConversationMessagesEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ListConversationMessagesRequest(
                    contact_id=contact_id,
                    conversation_id=conversation_id,
                    app_id=app_id,
                    page_size=page_size,
                    page_token=page_token,
                    view=view,
                    messages_source=messages_source,
                    only_recipient_originated=only_recipient_originated
                )
            )
        )


class ConversationApp:
    def __init__(self, sinch):
        self._sinch = sinch

    def create(
        self,
        display_name: str,
        channel_credentials: list,
        conversation_metadata_report_view: str = None,
        retention_policy: dict = None,
        dispatch_retention_policy: dict = None,
        processing_mode: str = None
    ) -> CreateConversationAppResponse:
        """
        Creates a new Conversation API app with one or more configured channels.
        The ID of the app is generated at creation and is returned in the response.
        https://developers.sinch.com/docs/conversation/api-reference/conversation/tag/App/#tag/App/operation/App_CreateApp
        """
        return self._sinch.configuration.transport.request(
            CreateConversationAppEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=CreateConversationAppRequest(
                    display_name=display_name,
                    channel_credentials=channel_credentials,
                    conversation_metadata_report_view=conversation_metadata_report_view,
                    retention_policy=retention_policy,
                    dispatch_retention_policy=dispatch_retention_policy,
                    processing_mode=processing_mode
                )
            )
        )

    def delete(self, app_id: str) -> DeleteConversationAppResponse:
        """
        Deletes the app identified by the app_id.
        """
        return self._sinch.configuration.transport.request(
            DeleteConversationAppEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=DeleteConversationAppRequest(app_id)
            )
        )

    def list(self) -> ListConversationAppsResponse:
        """
        Lists all apps for the project identified by the project_id.
        Returns the information as an array of app objects in the response.
        """
        return self._sinch.configuration.transport.request(
            ListAppsEndpoint(
                project_id=self._sinch.configuration.project_id
            )
        )

    def get(self, app_id: str) -> GetConversationAppResponse:
        """
        Returns the configuration information of the app, specified by the app_id, in the response.
        """
        return self._sinch.configuration.transport.request(
            GetAppEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=GetConversationAppRequest(
                    app_id=app_id
                )
            )
        )

    def update(
        self,
        app_id: str,
        display_name: str,
        channel_credentials: list = None,
        update_mask=None,
        conversation_metadata_report_view=None,
        retention_policy=None,
        dispatch_retention_policy=None,
        processing_mode=None
    ) -> UpdateConversationAppResponse:
        """
        Updates an existing Conversation API app with new configuration options defined in the request.
        The details of the updated app are returned in the response.
        """
        return self._sinch.configuration.transport.request(
            UpdateConversationAppEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=UpdateConversationAppRequest(
                    app_id=app_id,
                    display_name=display_name,
                    channel_credentials=channel_credentials,
                    update_mask=update_mask,
                    conversation_metadata_report_view=conversation_metadata_report_view,
                    retention_policy=retention_policy,
                    dispatch_retention_policy=dispatch_retention_policy,
                    processing_mode=processing_mode
                )
            )
        )


class ConversationContact:
    def __init__(self, sinch):
        self._sinch = sinch

    def update(
        self,
        contact_id: str,
        channel_identities: List[SinchConversationChannelIdentities] = None,
        language: str = None,
        display_name: str = None,
        email: str = None,
        external_id: str = None,
        metadata: str = None,
        channel_priority: list = None
    ) -> UpdateConversationContactResponse:
        """
        Updates an existing Conversation API contact with new configuration options defined in the request.
        The details of the updated contact are returned in the response.
        """
        return self._sinch.configuration.transport.request(
            UpdateConversationContactEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=UpdateConversationContactRequest(
                    channel_identities=channel_identities,
                    language=language,
                    display_name=display_name,
                    email=email,
                    external_id=external_id,
                    metadata=metadata,
                    channel_priority=channel_priority,
                    id=contact_id
                )
            )
        )

    def create(
        self,
        channel_identities: List[SinchConversationChannelIdentities],
        language: str,
        display_name: str = None,
        email: str = None,
        external_id: str = None,
        metadata: str = None,
        channel_priority: list = None
    ) -> CreateConversationContactResponse:
        """
        Creates a new Conversation API contact.
        The ID of the contact is generated at creation and is returned in the response.
        """
        return self._sinch.configuration.transport.request(
            CreateConversationContactEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=CreateConversationContactRequest(
                    channel_identities=channel_identities,
                    language=language,
                    display_name=display_name,
                    email=email,
                    external_id=external_id,
                    metadata=metadata,
                    channel_priority=channel_priority
                )
            )
        )

    def delete(self, contact_id: str) -> DeleteConversationContactResponse:
        """
        Deletes the Conversation API contact identified by the contact_id.
        """
        return self._sinch.configuration.transport.request(
            DeleteContactEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=DeleteConversationContactRequest(
                    contact_id=contact_id
                )
            )
        )

    def get(self, contact_id: str) -> GetConversationContactResponse:
        """
        Returns the configuration information of the
        Conversation API contact, specified by the contact_id, in the response.
        """
        return self._sinch.configuration.transport.request(
            GetContactEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=GetConversationContactRequest(
                    contact_id=contact_id
                )
            )
        )

    def list(
        self,
        page_size: int = None,
        page_token: str = None,
        external_id: str = None,
        channel: str = None,
        identity: str = None
    ) -> ListConversationContactsResponse:
        """
        Lists all Conversation API contacts for the project identified by the project_id.
        Returns the information as an array of contact objects in the response.
        """
        return TokenBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListContactsEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ListConversationContactRequest(
                    page_size=page_size,
                    page_token=page_token,
                    external_id=external_id,
                    channel=channel,
                    identity=identity
                )
            )
        )

    def merge(
        self,
        source_id: str,
        destination_id: str,
        strategy: str = None
    ) -> MergeConversationContactsResponse:
        """
        Merges two existing Conversation API contacts.
        The contact specified by the destination_id will be kept.
        The contact specified by the source_id will be deleted.
        All conversations from source contact are merged into destination.
        Channel identities and optional fields from source contact are only
        merged if corresponding entries do not exist in destination.
        """
        return self._sinch.configuration.transport.request(
            MergeConversationContactsEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=MergeConversationContactsRequest(
                    destination_id=destination_id,
                    strategy=strategy,
                    source_id=source_id
                )
            )
        )

    def get_channel_profile(
        self,
        app_id: str,
        recipient: SinchConversationRecipient,
        channel: ConversationChannel,
    ) -> GetConversationChannelProfileResponse:
        """
        Returns the user profile information for the specified recipient on the specified channel.
        This request is not supported for all Conversation API channels.
        """
        return self._sinch.configuration.transport.request(
            GetChannelProfileEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=GetConversationChannelProfileRequest(
                    app_id=app_id,
                    recipient=recipient,
                    channel=channel
                )
            )
        )


class ConversationContactWithAsyncPagination(ConversationContact):
    async def list(
        self,
        page_size: int = None,
        page_token: str = None,
        external_id: str = None,
        channel: str = None,
        identity: str = None
    ) -> ListConversationContactsResponse:
        return await AsyncTokenBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListContactsEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ListConversationContactRequest(
                    page_size=page_size,
                    page_token=page_token,
                    external_id=external_id,
                    channel=channel,
                    identity=identity
                )
            )
        )


class ConversationEvent:
    def __init__(self, sinch):
        self._sinch = sinch

    def send(
        self,
        app_id: str,
        recipient: dict,
        event: dict,
        callback_url: str = None,
        channel_priority_order: str = None,
        event_metadata: str = None,
        queue: str = None
    ) -> SendConversationEventResponse:
        return self._sinch.configuration.transport.request(
            SendEventEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=SendConversationEventRequest(
                    app_id=app_id,
                    recipient=recipient,
                    event=event,
                    callback_url=callback_url,
                    channel_priority_order=channel_priority_order,
                    event_metadata=event_metadata,
                    queue=queue
                )
            )
        )


class ConversationTranscoding:
    def __init__(self, sinch):
        self._sinch = sinch

    def transcode_message(
        self,
        app_id: str,
        app_message: dict,
        channels: list,
        from_: str = None,
        to: str = None
    ) -> TranscodeConversationMessageResponse:
        return self._sinch.configuration.transport.request(
            TranscodeMessageEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=TranscodeConversationMessageRequest(
                    app_id=app_id,
                    app_message=app_message,
                    channels=channels,
                    from_=from_,
                    to=to
                )
            )
        )


class ConversationOptIn:
    def __init__(self, sinch):
        self._sinch = sinch

    def register(
        self,
        app_id: str,
        channels: list,
        recipient: dict,
        request_id: str = None,
        processing_strategy: str = None
    ) -> RegisterConversationOptInResponse:
        return self._sinch.configuration.transport.request(
            RegisterOptInEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=RegisterConversationOptInRequest(
                    app_id=app_id,
                    recipient=recipient,
                    channels=channels,
                    request_id=request_id,
                    processing_strategy=processing_strategy
                )
            )
        )


class ConversationOptOut:
    def __init__(self, sinch):
        self._sinch = sinch

    def register(
        self,
        app_id: str,
        channels: list,
        recipient: dict,
        request_id: str = None,
        processing_strategy: str = None
    ) -> RegisterConversationOptOutResponse:
        return self._sinch.configuration.transport.request(
            RegisterOptOutEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=RegisterConversationOptOutRequest(
                    app_id=app_id,
                    recipient=recipient,
                    channels=channels,
                    request_id=request_id,
                    processing_strategy=processing_strategy
                )
            )
        )


class ConversationCapability:
    def __init__(self, sinch):
        self._sinch = sinch

    def query(
        self,
        app_id: str,
        recipient: dict,
        request_id: str = None
    ) -> QueryConversationCapabilityResponse:
        return self._sinch.configuration.transport.request(
            CapabilityQueryEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=QueryConversationCapabilityRequest(
                    app_id=app_id,
                    recipient=recipient,
                    request_id=request_id
                )
            )
        )


class ConversationTemplate:
    def __init__(self, sinch):
        self._sinch = sinch

    def create(
        self,
        translations: list,
        default_translation: str,
        channel: str = None,
        create_time: str = None,
        description: str = None,
        id: str = None,
        update_time: str = None
    ) -> CreateConversationTemplateResponse:
        return self._sinch.configuration.transport.request(
            CreateTemplateEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=CreateConversationTemplateRequest(
                    channel=channel,
                    create_time=create_time,
                    description=description,
                    id=id,
                    translations=translations,
                    default_translation=default_translation,
                    update_time=update_time
                )
            )
        )

    def list(self) -> ListConversationTemplatesResponse:
        return self._sinch.configuration.transport.request(
            ListTemplatesEndpoint(
                project_id=self._sinch.configuration.project_id
            )
        )

    def get(self, template_id: str) -> GetConversationTemplateResponse:
        return self._sinch.configuration.transport.request(
            GetTemplatesEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=GetConversationTemplateRequest(
                    template_id=template_id
                )
            )
        )

    def update(
        self,
        template_id: str,
        translations: list,
        default_translation: str,
        id: str = None,
        update_mask: str = None,
        channel: str = None,
        create_time: str = None,
        description: str = None,
        update_time: str = None
    ) -> UpdateConversationTemplateResponse:
        return self._sinch.configuration.transport.request(
            UpdateTemplateEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=UpdateConversationTemplateRequest(
                    channel=channel,
                    create_time=create_time,
                    description=description,
                    id=id,
                    translations=translations,
                    default_translation=default_translation,
                    update_time=update_time,
                    update_mask=update_mask,
                    template_id=template_id
                )
            )
        )

    def delete(self, template_id: str) -> DeleteConversationTemplateResponse:
        return self._sinch.configuration.transport.request(
            DeleteTemplateEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=DeleteConversationTemplateRequest(
                    template_id=template_id
                )
            )
        )


class ConversationWebhook:
    def __init__(self, sinch):
        self._sinch = sinch

    def create(
        self,
        app_id: str,
        target: str,
        triggers: list,
        client_credentials: dict = None,
        secret: str = None,
        target_type: str = None
    ) -> CreateWebhookResponse:
        return self._sinch.configuration.transport.request(
            CreateWebhookEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=CreateConversationWebhookRequest(
                    app_id=app_id,
                    target=target,
                    triggers=triggers,
                    client_credentials=client_credentials,
                    secret=secret,
                    target_type=target_type
                )
            )
        )

    def update(
        self,
        webhook_id: str,
        app_id: str,
        target: str,
        triggers: list,
        update_mask: str = None,
        client_credentials: dict = None,
        secret: str = None,
        target_type: str = None
    ) -> UpdateWebhookResponse:
        return self._sinch.configuration.transport.request(
            UpdateWebhookEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=UpdateConversationWebhookRequest(
                    app_id=app_id,
                    target=target,
                    triggers=triggers,
                    client_credentials=client_credentials,
                    secret=secret,
                    target_type=target_type,
                    update_mask=update_mask,
                    webhook_id=webhook_id
                )
            )
        )

    def list(self, app_id: str) -> SinchListWebhooksResponse:
        return self._sinch.configuration.transport.request(
            ListWebhooksEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ListConversationWebhookRequest(
                    app_id=app_id
                )
            )
        )

    def get(self, webhook_id: str) -> GetWebhookResponse:
        return self._sinch.configuration.transport.request(
            GetWebhookEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=GetConversationWebhookRequest(
                    webhook_id=webhook_id
                )
            )
        )

    def delete(self, webhook_id: str) -> SinchDeleteWebhookResponse:
        return self._sinch.configuration.transport.request(
            DeleteWebhookEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=DeleteConversationWebhookRequest(
                    webhook_id=webhook_id
                )
            )
        )


class ConversationConversation:
    def __init__(self, sinch):
        self._sinch = sinch

    def create(
        self,
        id: str = None,
        metadata: str = None,
        conversation_metadata: dict = None,
        contact_id: str = None,
        app_id: str = None,
        active_channel: str = None,
        active: bool = None,
    ) -> SinchCreateConversationResponse:
        return self._sinch.configuration.transport.request(
            CreateConversationEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=CreateConversationRequest(
                    app_id=app_id,
                    contact_id=contact_id,
                    id=id,
                    metadata=metadata,
                    conversation_metadata=conversation_metadata,
                    active_channel=active_channel,
                    active=active
                )
            )
        )

    def list(
        self,
        only_active: bool,
        page_size: int = None,
        page_token: str = None,
        app_id: str = None,
        contact_id: str = None
    ) -> SinchListConversationsResponse:
        return TokenBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListConversationsEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ListConversationsRequest(
                    only_active=only_active,
                    page_size=page_size,
                    page_token=page_token,
                    app_id=app_id,
                    contact_id=contact_id
                )
            )
        )

    def get(self, conversation_id: str) -> SinchGetConversationResponse:
        return self._sinch.configuration.transport.request(
            GetConversationEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=GetConversationRequest(
                    conversation_id=conversation_id
                )
            )
        )

    def delete(self, conversation_id: str) -> SinchDeleteConversationResponse:
        return self._sinch.configuration.transport.request(
            DeleteConversationEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=DeleteConversationRequest(
                    conversation_id=conversation_id
                )
            )
        )

    def update(
        self,
        conversation_id: str,
        update_mask: str = None,
        metadata_update_strategy: str = None,
        metadata: str = None,
        conversation_metadata: dict = None,
        contact_id: str = None,
        app_id: str = None,
        active_channel: str = None,
        active: bool = None
    ) -> SinchUpdateConversationResponse:
        return self._sinch.configuration.transport.request(
            UpdateConversationEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=UpdateConversationRequest(
                    app_id=app_id,
                    contact_id=contact_id,
                    conversation_id=conversation_id,
                    metadata=metadata,
                    conversation_metadata=conversation_metadata,
                    active_channel=active_channel,
                    active=active,
                    metadata_update_strategy=metadata_update_strategy,
                    update_mask=update_mask
                )
            )
        )

    def stop(self, conversation_id: str) -> SinchStopConversationResponse:
        return self._sinch.configuration.transport.request(
            StopConversationEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=StopConversationRequest(
                    conversation_id=conversation_id
                )
            )
        )

    def inject_message_to_conversation(
        self,
        conversation_id: str,
        accept_time: str = None,
        app_message: dict = None,
        channel_identity: dict = None,
        contact_id: str = None,
        contact_message: dict = None,
        direction: str = None,
        metadata: str = None
    ) -> SinchInjectMessageResponse:
        return self._sinch.configuration.transport.request(
            InjectMessageToConversationEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=InjectMessageToConversationRequest(
                    conversation_id=conversation_id,
                    accept_time=accept_time,
                    app_message=app_message,
                    channel_identity=channel_identity,
                    contact_id=contact_id,
                    contact_message=contact_message,
                    direction=direction,
                    metadata=metadata
                )
            )
        )


class ConversationConversationWithAsyncPagination(ConversationConversation):
    async def list(
        self,
        only_active: bool,
        page_size: int = None,
        page_token: str = None,
        app_id: str = None,
        contact_id: str = None
    ) -> SinchListConversationsResponse:
        return await AsyncTokenBasedPaginator._initialize(
            sinch=self._sinch,
            endpoint=ListConversationsEndpoint(
                project_id=self._sinch.configuration.project_id,
                request_data=ListConversationsRequest(
                    only_active=only_active,
                    page_size=page_size,
                    page_token=page_token,
                    app_id=app_id,
                    contact_id=contact_id
                )
            )
        )


class ConversationBase:
    """
    Documentation for the Conversation API: https://developers.sinch.com/docs/conversation/
    """

    def __init__(self, sinch):
        self._sinch = sinch


class Conversation(ConversationBase):
    """
    Synchronous version of the Conversation Domain
    """
    __doc__ += ConversationBase.__doc__

    def __init__(self, sinch):
        super(Conversation, self).__init__(sinch)
        self.message = ConversationMessage(self._sinch)
        self.app = ConversationApp(self._sinch)
        self.contact = ConversationContact(self._sinch)
        self.event = ConversationEvent(self._sinch)
        self.transcoding = ConversationTranscoding(self._sinch)
        self.opt_in = ConversationOptIn(self._sinch)
        self.opt_out = ConversationOptOut(self._sinch)
        self.capability = ConversationCapability(self._sinch)
        self.template = ConversationTemplate(self._sinch)
        self.webhook = ConversationWebhook(self._sinch)
        self.conversation = ConversationConversation(self._sinch)


class ConversationAsync(ConversationBase):
    """
    Asynchronous version of the Conversation Domain
    """
    __doc__ += ConversationBase.__doc__

    def __init__(self, sinch):
        super(ConversationAsync, self).__init__(sinch)
        self.message = ConversationMessageWithAsyncPagination(self._sinch)
        self.app = ConversationApp(self._sinch)
        self.contact = ConversationContactWithAsyncPagination(self._sinch)
        self.event = ConversationEvent(self._sinch)
        self.transcoding = ConversationTranscoding(self._sinch)
        self.opt_in = ConversationOptIn(self._sinch)
        self.opt_out = ConversationOptOut(self._sinch)
        self.capability = ConversationCapability(self._sinch)
        self.template = ConversationTemplate(self._sinch)
        self.webhook = ConversationWebhook(self._sinch)
        self.conversation = ConversationConversationWithAsyncPagination(self._sinch)



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.batches.responses import SendSMSDeliveryFeedbackResponse
from sinch.domains.sms.models.batches.requests import SendDeliveryFeedbackRequest


class SendDeliveryReportEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/batches/{batch_id}/delivery_feedback"
    HTTP_METHOD = HTTPMethods.POST.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: SendDeliveryFeedbackRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id,
            batch_id=self.request_data.batch_id
        )

    def request_body(self):
        self.request_data.batch_id = None
        return self.request_data.as_json()

    def handle_response(self, response: HTTPResponse):
        super(SendDeliveryReportEndpoint, self).handle_response(response)
        return SendSMSDeliveryFeedbackResponse()



from dataclasses import dataclass

from sinch.core.models.base_model import SinchBaseModel


@dataclass
class ConversationWebhook(SinchBaseModel):
    id: str
    app_id: str
    target: str
    target_type: str
    secret: str
    triggers: list
    client_credentials: dict



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.inbounds import InboundMessage
from sinch.domains.sms.models.inbounds.requests import ListSMSInboundMessageRequest
from sinch.domains.sms.models.inbounds.responses import SinchListInboundMessagesResponse


class ListInboundMessagesEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/inbounds"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: ListSMSInboundMessageRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id
        )

    def build_query_params(self):
        return self.request_data.as_dict()

    def handle_response(self, response: HTTPResponse):
        super(ListInboundMessagesEndpoint, self).handle_response(response)
        return SinchListInboundMessagesResponse(
            inbounds=[
                InboundMessage(
                    type=inbound.get("type"),
                    id=inbound.get("id"),
                    from_=inbound.get("from"),
                    to=inbound.get("to"),
                    body=inbound.get("body"),
                    operator_id=inbound.get("operator_id"),
                    send_at=inbound.get("send_at"),
                    received_at=inbound.get("received_at"),
                    client_reference=inbound.get("client_reference")
                ) for inbound in response.body["inbounds"]
            ],
            page=response.body.get("page"),
            page_size=response.body.get("page_size"),
            count=response.body.get("count")
        )



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.groups.requests import ListSMSGroupRequest
from sinch.domains.sms.models.groups.responses import SinchListSMSGroupResponse
from sinch.domains.sms.models.groups import SMSGroup


class ListSMSGroupEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/groups"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: ListSMSGroupRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.sms_origin,
            project_or_service_id=self.project_or_service_id
        )

    def build_query_params(self):
        return self.request_data.as_dict()

    def handle_response(self, response: HTTPResponse):
        super(ListSMSGroupEndpoint, self).handle_response(response)
        return SinchListSMSGroupResponse(
            groups=[
                SMSGroup(
                    id=group.get("id"),
                    size=group.get("size"),
                    created_at=group.get("created_at"),
                    modified_at=group.get("modified_at"),
                    name=group.get("name"),
                    child_groups=group.get("child_groups"),
                    auto_update=response.body.get("auto_update")
                ) for group in response.body["groups"]
            ],
            page=response.body.get("page"),
            page_size=response.body.get("page_size"),
            count=response.body.get("count")
        )



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class CreateConversationTemplateRequest(SinchRequestBaseModel):
    channel: str
    create_time: str
    description: str
    id: str
    translations: list
    default_translation: str
    update_time: str


@dataclass
class GetConversationTemplateRequest(SinchRequestBaseModel):
    template_id: str


@dataclass
class DeleteConversationTemplateRequest(SinchRequestBaseModel):
    template_id: str


@dataclass
class UpdateConversationTemplateRequest(CreateConversationTemplateRequest):
    update_mask: str
    template_id: str



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.voice.endpoints.voice_endpoint import VoiceEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.voice.models.conferences.responses import KickAllVoiceConferenceResponse
from sinch.domains.voice.models.conferences.requests import KickAllVoiceConferenceRequest


class KickAllConferenceEndpoint(VoiceEndpoint):
    ENDPOINT_URL = "{origin}/calling/v1/conferences/id/{conference_id}"
    HTTP_METHOD = HTTPMethods.DELETE.value
    HTTP_AUTHENTICATION = HTTPAuthentication.SIGNED.value

    def __init__(self, request_data: KickAllVoiceConferenceRequest):
        self.request_data = request_data

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.voice_origin,
            conference_id=self.request_data.conference_id
        )

    def handle_response(self, response: HTTPResponse) -> KickAllVoiceConferenceResponse:
        super().handle_response(response)
        return KickAllVoiceConferenceResponse()



from sinch.core.models.http_response import HTTPResponse
from sinch.domains.sms.endpoints.sms_endpoint import SMSEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.sms.models.batches import Batch
from sinch.domains.sms.models.batches.requests import ListBatchesRequest
from sinch.domains.sms.models.batches.responses import ListSMSBatchesResponse


class ListSMSBatchesEndpoint(SMSEndpoint):
    ENDPOINT_URL = "{origin}/xms/v1/{project_or_service_id}/batches"
    HTTP_METHOD = HTTPMethods.GET.value
    HTTP_AUTHENTICATION = HTTPAuthentication.OAUTH.value

    def __init__(self, request_data: ListBatchesRequest, sinch):
        super().__init__(request_data, sinch)

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=self.sms_origin,
            project_or_service_id=self.project_or_service_id
        )

    def build_query_params(self):
        return self.request_data.as_dict()

    def handle_response(self, response: HTTPResponse):
        super(ListSMSBatchesEndpoint, self).handle_response(response)
        return ListSMSBatchesResponse(
            batches=[
                Batch(
                    id=batch.get("id"),
                    to=batch.get("to"),
                    from_=batch.get("from"),
                    body=batch.get("body"),
                    delivery_report=batch.get("delivery_report"),
                    cancelled=batch.get("cancelled"),
                    type=batch.get("type"),
                    campaign_id=batch.get("campaign_id"),
                    created_at=batch.get("created_at"),
                    modified_at=batch.get("modified_at"),
                    send_at=batch.get("send_at"),
                    expire_at=batch.get("expire_at")
                ) for batch in response.body["batches"]
            ],
            page=response.body.get("page"),
            page_size=response.body.get("page_size"),
            count=response.body.get("count")
        )



from datetime import datetime


def timestamp_to_datetime_in_utc_deserializer(timestamp: str):
    """
    Older Python versions (like 3.9) do not support "Z" as a TZ information.
    One needs to use '+00:00' to represent UTC tz.
    """
    if timestamp.endswith("Z"):
        timestamp = timestamp[:-1]

    return datetime.fromisoformat(timestamp + "+00:00")



from abc import ABC, abstractmethod


class AuthenticationBase(ABC):
    def __init__(self, sinch):
        self.sinch = sinch

    @abstractmethod
    def get_auth_token(self):
        pass

    @abstractmethod
    def set_auth_token(self, token):
        pass


class Authentication(AuthenticationBase):
    def get_auth_token(self):
        return self.sinch.configuration.token_manager.get_auth_token()

    def set_auth_token(self, token):
        self.sinch.configuration.token_manager.set_auth_token(token)


class AuthenticationAsync(AuthenticationBase):
    async def get_auth_token(self):
        return await self.sinch.configuration.token_manager.get_auth_token()

    async def set_auth_token(self, token):
        return await self.sinch.configuration.token_manager.set_auth_token()



import json
from sinch.core.models.http_response import HTTPResponse
from sinch.domains.voice.endpoints.voice_endpoint import VoiceEndpoint
from sinch.core.enums import HTTPAuthentication, HTTPMethods
from sinch.domains.voice.models.applications.requests import UnassignNumbersVoiceApplicationRequest
from sinch.domains.voice.models.applications.responses import UnassignNumbersVoiceApplicationResponse


class UnAssignVoiceNumberEndpoint(VoiceEndpoint):
    ENDPOINT_URL = "{origin}/v1/configuration/numbers"
    HTTP_METHOD = HTTPMethods.DELETE.value
    HTTP_AUTHENTICATION = HTTPAuthentication.SIGNED.value

    def __init__(self, request_data: UnassignNumbersVoiceApplicationRequest):
        self.request_data = request_data

    def build_url(self, sinch) -> str:
        return self.ENDPOINT_URL.format(
            origin=sinch.configuration.voice_applications_origin
        )

    def request_body(self):
        request_data = {}

        if self.request_data.number:
            request_data["number"] = self.request_data.number

        if self.request_data.application_key:
            request_data["applicationKey"] = self.request_data.application_key

        if self.request_data.capability:
            request_data["capability"] = self.request_data.capability

        return json.dumps(request_data)

    def handle_response(self, response: HTTPResponse) -> UnassignNumbersVoiceApplicationResponse:
        super().handle_response(response)
        return UnassignNumbersVoiceApplicationResponse()



from dataclasses import dataclass
from sinch.core.models.base_model import SinchRequestBaseModel


@dataclass
class CreateSMSGroupRequest(SinchRequestBaseModel):
    name: str
    members: list
    child_groups: list
    auto_update: dict


@dataclass
class ListSMSGroupRequest(SinchRequestBaseModel):
    page_size: int
    page: int


@dataclass
class DeleteSMSGroupRequest(SinchRequestBaseModel):
    group_id: str


@dataclass
class GetSMSGroupRequest(SinchRequestBaseModel):
    group_id: str


@dataclass
class GetSMSGroupPhoneNumbersRequest(SinchRequestBaseModel):
    group_id: str


@dataclass
class UpdateSMSGroupRequest(SinchRequestBaseModel):
    group_id: str
    name: str
    add: list
    remove: list
    auto_update: dict
    add_from_group: str
    remove_from_group: str


@dataclass
class ReplaceSMSGroupPhoneNumbersRequest(SinchRequestBaseModel):
    group_id: str
    members: list
    name: str
